#
# -*- awk -*-

# This file first checks that the change log is up to date
# if so, it builds the release change log
# and builds a version file for the other build tools to 
# use in naming the components.
#
# we assume fr (the main) is passed and therefor open.
BEGIN { 
version = ""
}

firstfile == "" {firstfile = FILENAME}

firstfile == FILENAME {
    while ( NR < 9 ) {
	ver = gensub(/.+<20([0-9][0-9])([0-9][0-9])([0-9][0-9]).([0-9][0-9]).+/, "\\1.\\2.\\3.\\4", "g")
	if ( ver != $0 ) {
	    #gsub(/\.0/,".",ver)
	    version = ver
	    print "Current version is "version
	    break
	}
	getline
    }
    if (version == "") {
	print "Failed to find version in first 10 lines of "FILENAME".  Fix this"
	exit 1
    }
    nextfile
}
#/.*/ {print "processing "FILENAME " begining with line "FNR" first file was "firstfile}
firstfile != FILENAME {
    # Second file, now we need to find the revcode...
    r[FNR] = $0
    while ( FNR < 10 ) {
#	print "testing "$0
	if (match($0,/.*[0-9][0-9].[0-9][0-9]*.[0-9][0-9]*.*/)) {
	    ver2 = gensub(/.*([0-9][0-9].[0-9][0-9]*.[0-9][0-9]*.[0-9][0-9]*).*/, "\\1","g")
	    if ( ver2 != version) {
		print "Version mismatch, \n"firstfile" version "version",\n" FILENAME" version "ver2"\n fix this!"
		exit 1
	    }
	    print "Found version "ver2 " in "FILENAME" building change log"
	    system ("mkdir -p " "../release-"ver2)
	    for (i = 1; i <= FNR; i++) { 
		print r[i] >"../release-"ver2"/changelog-"ver2".txt"
	    }
	    while (getline > 0) {
		print $0 >"../release-"ver2"/changelog-"ver2".txt"
		if (match($0,/^--+.?$/) && length($0)>30) {
		    break
		}
	    }
	    print ver2 >"version"
	    print "Indexing the Tip file"
	    try_this("./tip_index.tcl ../doc/Tips.txt")
	    print "wraping filerunner"
	    try_this( "./wrapwindows " ver2)
	    print "building the tar file"
	    try_this( "./pack " ver2 )
	    print "building the RPM and DEB files"
	    try_this("./buildRPM.tcl -v EMAIL=tom@wildturkeyranch.net NAME=\"Tom Turkey\" TAR=../release-"ver2"/fr-"ver2".tar.gz" )

	    print "Moving the RPM and DEB files to ../release-"ver2

	    try_this("mv ~/rpmbuild/RPMS/noarch/*"ver2"* ../release-"ver2)
	    try_this("mv ~/rpmbuild/DEB/*"ver2"*.deb ../release-"ver2)
	    try_this("mv ~/rpmbuild/SRPMS/*"ver2"* ../release-"ver2)

# The following 3 lines are unique to my system.  They should not bother you
# unless you have a system named "raspi" on your local network.

	    if ( system("ping -w 1 -c 1 raspi >&/dev/null") == 0 ) {
		try_this("rsync -av ../release-"ver2" raspi:/fa/filerunner")
            }

	    print "success version "ver2 " made and ready"
	    print "Time to test."
	    exit 0
	    break
	}
	next
    }
    print "Failed to find version in first 10 lines of history ("FILENAME") file.  Fix this."
    exit 0
}
function try_this( what ) {
    if ( system(what) != 0 ) {
	print "Error trying to execute: " what
	print "Stop the music."
	exit 1
    }
} 
END {
}
