#
# AddConf does most of the writting of the configure file with all the comments.
#
# parameters:
#    fid      The file to 'put' to
#    var_var  The name of the variable we are to write instructions to set.
#             If var_var is a list e.g. {config menu,*} it is used to get 
#             an array of entries to set up. In this case, all config(menu,*)
#             entries.
#    setto    is a flag with values 
#             ""  -> single line set statment
#             1   -> print one line per each level entry in the list
#             2   -> print one line per first rest on second line
#             "{" -> print 'set var {' caller to complete...
#    expl     This is the comment to be printed prior to the set statment.
#             If not empty, a line of "-----" is first printed to seperate 
#             the text from the prior entry in the file.
#
proc AddConf { fid var_var setto expl } {
  global glob config 
  if { $expl != "" } {
    puts -nonewline $fid "#-----------------------------------------------------------------------"
    #-----------------------------------------------------------------------
    #puts -nonewline $fid "# Variable: $var_var"
    puts $fid "$expl"
  }
  lassign $var_var arrayname pattern 
  set r [catch {array get $arrayname $pattern} varList]
  if {$r == 0 && $varList !={}} {
    set var_var {}
    foreach {ent dum} $varList {
      lappend var_var ${arrayname}($ent)
    }
  }
#    puts "$var_var"
  foreach val $var_var {
    #  upvar $var_var var
    upvar $val var
    AddConfPutVar
  }
}

proc AddConfPutVar {} {
  uplevel {
    puts  -nonewline $fid "set $val"
    if { $setto == "" } {
      if {[string match "* *" $var] || $var == {}} {
	puts $fid " \{$var\}"
      } else {
	puts $fid " $var"
      }
    } elseif { $setto == "1" || $setto == "2"} {
      # make single line per first level entry
      puts $fid " \{"
      foreach kk $var {
	puts -nonewline $fid "    "
	if {$setto == "2"} {
	  if {[string match "* *" $kk] || $kk == {} } {
	    puts $fid "\{$kk\}"
	  } else {
	    puts $fid "$kk"
	  }
	} else {
	  if {[set t [llength $kk]] != 1} {
	    puts -nonewline $fid "\{"
	  }
	  set in 0
	  foreach k $kk {
	    if {$in > 0} {
	      puts -nonewline $fid "\n        "
	    } 
	    if {[string match "* *" $k] || $k == {} } {
	      puts -nonewline $fid "\{$k\}"
	    } else {
	      puts -nonewline $fid "$k"
	    }
	    incr in
	  }
	  if {$t != 1} {
	    puts $fid "\}"
	  } else {
	    puts $fid ""
	  }
	}
      }
      puts $fid "  \}\n"
    } else {
      puts -nonewline $fid " $setto"
    }
    puts $fid ""
  }
}

proc ClearTheDecksHereComeANewConfig {} {
    global glob config
    # A call to this proc is in the front end of the config file.
    # What we need to do is eliminate any old stuff that it may not
    # overlay.  Mostly this will be array entries that have been eliminated.

    # Eliminate the old bind array so we don't pop it up after the user tried
    # to deep six it....
    # We don't do this for the menu array as it will not bit her/him if it 
    # comes back.  And, of course, changeing it will overlay what is.
    foreach {but val} [array get config "bind,*"] {
	unset config($but)
    }
}
    
proc SaveConfig {}  {
  global glob config env
  lassign [getOldNewVersions] oldv newv
  set r [catch {open $glob(conf_dir)/config$glob(cygwin)-$newv w} fid]
  if {$r != 0} { PopError "Saving config: $fid"; return "" }

  puts $fid \
"# Generated by FileRunner $glob(version) on [clock format [clock seconds]]"
  
  puts $fid {
# This is the configuration file for FileRunner. It is read every time
# FileRunner starts up. It is saved in your home directory as
# "~/.fr/config" (~ is short for the path to your home directory). Make
# sure to only edit values for variables that are listed. Do not add or
# delete anything, since this file will be automatically regenerated
# when you do a "Edit Configuration->Save Configuration" in the menu and those
# changes will be lost.
#
# This config file is saved to disk and reread into FileRunner when you
# press the OK button in the config browser. You can also edit this file
# with a regular text editor and re-read it into FileRunner with
# "Configuration->Reread Configuration"

ClearTheDecksHereComeANewConfig

#**************** HOTLISTS **********************************************
}

  AddConf $fid config(hotlist) "{" {
# This list is what will show up in your HotList menu. (Like "bookmarks"
# in Netscape). Feel free to edit this list.
#
# You can label your selections thusly: {label /home/usr/foobar} 
# and you can build a one level cascade menu thusly: {-clabel { a b c }}
# where each 'a' 'b' 'c' is an entry which also may have a label as above.
# The '-' indicates that this a label for a cascade menu.  
# For example here is an entry for the Windows volumes:
# {-volumes {{floppy a:/} {{system volume} c:/} { d:/ }}}
# The '{}' around an entry allow spaces which otherwise seperate the entries.
# Try it...
}

  foreach k $glob(hotlist) {
    if { [lindex $k 1] != "" } {
      puts $fid "    {$k}"
    } else {
      puts $fid "    $k"
    }
  }
  puts $fid "  } \n"

  AddConf $fid config(runlist) 1 {
# This list is used by the Run ... commands (Utilities menu).  It contains
# a list of programs that you can select and instructions on how to run them.
# You may find it easier to add to this list by invoking the Run command
# and selecting 'Enter program name' where you can use command completion and
# or browsing to find the program name.
#
# Each entry in the list contains 2 to 4 entries as follows:
#
# The first entry is the display name (or 'nice' name) and the run string. 
# You may choose any thing you want for the display name.  The run string
# should contain the program name (possibly a full path name) followed by any
# flags you want to code.  The file(s) you are passing will replace the '%s'
# in this string. If the program name contains spaces you should can either
# enclose the full program name in curly braces ({}) or precede each space
# with a back slash (\).  Also, if on windows, use '/' instead of '\'
# in file names. 
#
# The second entry is the string you want to separate the files with 
# (should you ever pass more than one file).  Usually this is a space, coded as
# { }.
#
# The remaining two entries are 'async' and 'fullname' coded in any order.
# These are optional entries.  If 'async' is present the program is run
# asynchronously.  If 'fullname' is coded only full filenames are passed.
# Here is an example entry to run the 'thunderbird' mailer to compose a 
# mail message with the selected files as attachments. It will display as
# 'mail-to' when the 'Run' command is invoked.
#
#  {{mail-to {thunderbird -compose attachment='%s'}} , fullname async}
#
# If you need to use the quotes '"' use '\' to protect them e.g.'\"'.
}
  puts $fid {


#****** MOUSE & KEY BINDINGS & MENUS ***********************************
}
  AddConf $fid {} "" {
# This is the binding list. There are two types of bindings, global and 
# local to a directory.  Global bindings may be triggered anywhere in
# filerunner's main window, except that local bindings will suppress global
# bindings in the directory windows. So, for example mouse 3 can be locally 
# bound to ViewOne and globally bound to a menu. In a directory window mouse-3
# will do ViewOne, while outside of the directory it will bring up the menu. 
# Global bindings look like:

# config(global-bind,event) action

# local bindings look like:

# config(bind,event) action

# Each entry binds a key or mouse event to an action. 
# Legal actions are all the possible (not just those you configured
# to be present) commands in the center button list, including
# user defined buttons, plus the following commands from other filerunner 
# menus (listed below). Of these commands 'CmdRunCmd' 'Back' and 
# 'UpDirTree' take parameters. For 'CmdRunCmd' the first parameter is '1'
# if both selected and prior selected files are to be passed and the 
# second parameter is the 'nice' name of a program that must appear
# in the runlist (see above).  Both of these parameters are 
# optional. For example, using the 'mail-to' example in the runlist, this:
#
# set config(bind,Button-2) {CmdRunCmd 0 mail-to}
#
# will cause a mouse button 2 press to run thunderbird to bring up a mail 
# compose window with the selected files 'attached' to the message.
# The commands 'Back' and 'UpDirTree' must be coded exactly as shown.
# 'Back' pops a dir from the dir stack and makes it the current dir 
# (the same as the <- button).  'UpDirTree' is the same as the ^ button.
# One of these commands might be bound to the Left key (left arrow).  You
# then might want to bind the Right key (right arrow) to 'ViewOne'.
# The 'Toggle' command takes as a parameter a config item and toggles it.
# For example 'Toggle config(fileshow,all)' All the binary items in the 
# Configuration menu may be toggled this way.  Binary config options not
# in this menu will, most likely not work, especially if their changes require
# rebuilding the window (e.g. the keyboard support binary).
# Legal values for the 'bind' command may be found in the man page (do a
# google search for 'tk bind').  In general, mouse buttons 1, 2 & 3 with
# modifiers 'Double, Triple, Meta, Alt, Control etc. may be used. (Do
# understand that 'Double' & 'Triple' will also invoke the actions leading
# to the given action, that is a Triple will also invoke the single and double.
# Also, the button 1 selection will happen even if button one is also
# bound to a command (this is not recommended). Also note that all printing
# chars are used by the position command. If you do bind a printing character
# that binding as well as the position command will be executed.
#
# In the list below, the first element in each entry is the 'nice'
# label, the  second is the actual command name. You may use either one in
# bind and menu configuration. This is a list of the bindings filerunner
# ships with.  These may be changed or eliminated at your discretion:

#   set config(bind,Button-3) ViewOne
#   set config(bind,Double-1) {DoMenu,three}
#   set config(bind,Control-3) ViewDirOpposite
#   set config(bind,Control-1) {DoMenu,sample}
#   set config(bind,Button-2) {DoMenu,two}

# Here is the current list:
}
  foreach k $glob(buttoncmds) {
    if { [lindex $k 1] != "" || $k == {}} {
      puts $fid "#    {$k}"
    } else {
      puts $fid "#    $k"
    }
  }
  puts $fid "\n"
  AddConf $fid {config bind,*} "" {}
  AddConf $fid {config global-bind,*} "" {}
  AddConf $fid {config menu,*} "1" {
# These are the menus.  Each menu is a list of commands & menu references.
# a menu reference looks like 'DoMenu,name' where 'name' is the menu's 
# name and should appear here in a config(menu,name) description.
# You may define as many menus as you like, however, only menus that
# are referred to by one of the bindings listed above (and cascade menus
# of those) will be built. An empty entry (i.e. {}) will generate a
# separator in the menu.  All of these menus may be 'torn off' by clicking
# on the top entry (the '-----').  This promotes the menu to a window which
# may be moved around on the screen.  Torn off menus no longer have access
# to the list box entry that you were on when the menu was first popped up
# but they do have access to the current selection(s). To continue our 
# example of using the run command, here is a menu line that will give a
# menu entry called 'Mail To' which will run 'thunderbird' (see the runlist 
# above) to bring up a compose window with the selected files attached.
#
#    {{Mail To} {CmdRunCmd 0 mail-to}}
  }
  AddConf $fid config(no_selection) 1 {
# Commands in this list will not update the selection when they are 
# selected via a bind command.  All other commands will up date the selection
# if, when they are called via a 'bind'ing, there is no current selection.
# You should only add to this list if you really know what you are doing.
# These commands need to know how to work without a selection.
# Removing commands from this list will make the removed commands update the
# selection the same as others do.
  }
  AddConf $fid config(paste) 1 {
# The "paste" configuration item allow you to define additional mouse or
# key selections that do a "paste".  The standard paste characters
# <Control-v> and <Control-y> are already defined.  In addition some systems 
# define <Button2-ButtonRelease> and <Control-Shift-Insert>.  This info is
# required for paste operations in the directory line where the paste is
# intercepted to make magic happen. At the same time, any sequence named 
# here will result in a 'paste' being preformed (without the magic) in all
# filerunner 'view' windows.  If your system has other sequences for 'paste'
# but you don't used them to paste in the directory line you don't need to
# enter them here.  Filerunner is shipped with <Button2-ButtonRelease> set
# as this is common in the Unix world. Syntax is the same as the Tc/Tk
# bind command (see bind(n)).
  }
  puts $fid {
#****** COMMAND CONFIGURATION ******************************************
}



  AddConf $fid config(space) "" {
# The configuration elements below that contain '%s' are passed through
# a formater which causes the loss of any distinction between a space
# in a parameter (program name, file name, or what ever) and the space
# that separates them.  In order to retain this information this variable
# is used to replace the meaningful spaces prior to the format call and
# thus recover them after the format call.  In the wildly likely case
# that I have chosen a favorite character sequence of yours, here is
# your chance to change it.
#
# Please note that you are not required to use this sequence in the 
# below code, but just to indicate that spaces are meaningful in a
# character string by enclosing it in curly braces e.g. {hi there}
#
# While on the subject of quoting or escaping stings, the following
# characters, if they appear in one of these stings, must be quoted:
#  \[ { } $ "  Also, depending on the string use, it may be necessary to
# quote characters that have special meaning to the shell that the command 
# is passed to.  Quoting may be by {} as indicated above or by '\'.
# For example this string: $ID:.*Exp $ (used in my diff command) is quoted:
# {$ID:.*Exp $} to protect both the $ and the space. You could also use:
# \$ID:.*Exp\ \$ to get the same result.
  }

  AddConf $fid config(editor) "" {
# This variable should be set to the name of your favorite text
# editor. This editor will be invoked when you select files and press
# the "Edit" button. It is also invoked when you double click on any
# file name in a list box.  The text editor you choose should be an
# window application, not a text-terminal application. You can start
# text-terminal editors for example by launching them from inside an
# xterm. E.g: "xterm -e vi %s". The editor you choose should be able to
# load multiple files specified as arguments to the editor (e.g: 
# "emacs /tmp/foo /home/joe/bar"). The editor will be launched in the
# background.
# Note: If you use a path name and any elements of it contain blanks
# you will need to do something like:
# set config(editor) {{c:/Program Files/Windows NT/Accessories/wordpad.exe} %s}
# You could also just put the path in you PATH env var, either in your system
# environment set up or in the config file in filerunner's home directory:
# global env; set env(PATH) "$env(PATH):wordpad.exe" (linux/unix) or
# global env; set env(PATH) "$env(PATH);wordpad.exe" (windows).
}

  AddConf $fid config(view,extensions) "1" {
# This parameter contains a list of commands and the file patterns that
# are used when finding the right viewer for various files. The commands
# will receive a list of selected files. The first file in the selected
# list are used when deciding which viewer should be used. (Don't try
# and view a couple of text files and images simultaneously by selecting
# and pressing the view button). If you add the option "-viewtext"
# last in the rule that means the command outputs a listing/view and it
# should be viewed in the internal text viewer. The viewing commands
# will be executed in the background. View may be used to launch applications
# based on the file extension, however, for some files it makes more
# sense to launch the application is a 'peek' mode.  For example you
# may want to 'peek' inside a .zip file to see what it contains with
# { {unzip -lv %s} {*.zip} -viewtext }
# where the '-lv' tells unzip to list the files in the package.
# This is particularly true of files that are addressed by the UnArc and
# the UnPack buttons where, for example we might want
#  { {unzip -oqq %s} {*.zip} } 
# to actually unzip the file.
}

  AddConf $fid config(open,extensions) "1" {
# This parameter contains a list of commands and the file patterns that
# are used when finding the right 'open' routine for various files. The
# commands will receive a list of selected files. The first file in the
# selected list are used when deciding which opener should be used. (Don't
# try and open a couple of text files and images simultaneously by selecting
# and pressing the open button). If you add the option "-viewtext"
# last in the rule that means the command outputs a listing/view and it
# should be viewed in the internal text viewer. The opening commands
# will be executed in the background. Open is used to launch applications
# based on the file extension, however, for some files it makes more
# sense to launch the application in a 'peek' mode.  For this see the
# view command.
}

  AddConf $fid config(cmd,open) "" {
# This is command for the 'Open' command (above). For Windows this should be
# 'cmd /c'.  For linux I am aware of 'gnome-open' 'kde-open' and 
# 'xdg-open' (one file limit)  and 'gvfs-open' (runs program for each file 
# rather than send all files to the same program).
# Mac systems use 'open'. 
}
  
  AddConf $fid config(cmd,print) "" {
# The print command you want to use to print files.
}

  AddConf $fid config(cmd,term) "" {
# The name of your favorite terminal program.
# This is the program that will be launched when you press the button
# with the image of the terminal window (upper right of the file list
# panels). It will be launched in the background.  If you include a %s
# in this command it will be replaced with the current working dir 
# e.g. {konsole --workdir %s}  
# The command is also invoked in the current working dir.
}

  AddConf $fid config(cmd,diff) "" {
# Your favorite diff command. Should be able to diff directories also.
# Used in the "Diff" button command. This program will run
# synchronously, i.e. Filerunner will wait until it's done.
}

  puts $fid {
#****** ARCHIVER/PACKER CONFIGURATION **********************************
}

  AddConf $fid config(cmd,unarc,extensions) "1" {
# Extensions and bindings for unarchive rules. This is a list of
# commands and file name extensions. Filerunner will use this variable
# when deciding which command should be used to unarchive the file when
# you press the UnArc button. See above comments on the 'view' command.
# Its OK to put identical rules here and in the following 'unpack'
# command to, for example, unzip a file even if you push the wrong
# button.
}

  AddConf $fid config(cmd,unpack,extensions) "1" {
# Extensions and bindings for unpacking rules. This is a list of
# commands and file name extensions. Filerunner will use this variable
# when deciding which command should be used to unpack the file when you
# press the UnPack button.
}

  AddConf $fid config(cmd,pack) "" {
# The name of your favorite packer command. 
# Used in the "Arc" button command to pack files. Run synchronously
# or in the background.
}

  AddConf $fid config(cmd,archive) "" {
# This is the name of your archiver program. The archiver program is
# passed the name of the directory selected and is executed in the
# current directory. The archive created is supposed to contain the
# contents of the directory. The default built in "tar+gz %s" creates
# an archive named <directory>.tar.gz packed with tar and gzip. This
# command is used in the "Arc" button command to pack/archive
# directories. Can be run synchronously or in the background.
# This command takes up to three parameters (%s)'s.  The first is the
# selected file (or directory) the second is the other panels working
# directory and the third is again the selected file.  Thus:
# "tar -cz  %s -f %s/%s.tar.gz" is the same as "tar+gz %s" except
# that the result will be put in the 'other' directory.
}

  puts $fid {
#****** SFTP/FTP CONFIGURATION **********************************************
}

  AddConf $fid config(ftp,login) 1 {
# Rule based login information. Allows automatic determination
# if the login should use a proxy and what username/password it
# should use etc.
# See the online guide, chapter 8.8.
# Each entry in this list consists of the following:
# URL of the connection
# The user name and 'password info' 
# A proxy name.
# Any but the URL may be blank if not needed.
# The 'password info' has the following three parts:
# A password (may be {} for sftp connections using ssh keys)
# The disk address of the ssh private key (may be a full address or 
#    the file name if the key is in env(HOME)/.ssh)
# The pass phrase.
#
# If any of these entries contain blanks, enclose the whole
# entry in braces e.g.: {Hello world}
# Note that filerunner will encrypt the 'password info'.  If you need to change
# it you should delete the string: '>> list of numbers <<' and enter the new
# info in plain text.  File runner will encrypt it automatically.
}

  AddConf $fid config(ftp,fastlink) "" {
# Set this to 1 if you feel your FTP connections are fast, to 0
# otherwise. If you set it to 1, soft-links in FTP listings are
# correctly identified either as links to directories or to files,
# otherwise it will always think links are links to directories.
}

  AddConf $fid config(ftp,proxy) "" {
# If you have to go through a firewall via an FTP proxy, set this
# variable to the hostname of your FTP proxy. Set it to "" if you don't
# have an FTP proxy.
}

#  AddConf $fid config(ftp,user) "" {
# Set username for FTP sessions. If you are doing anonymous FTP, set
# this to "anonymous".
#} 

  AddConf $fid config(ftp,password) "" {
# Set password for anonymous FTP sessions. This should be your 
# email address.
}

  AddConf $fid config(ftp,timeout) "" {
# Set timeout in seconds for FTP communication. 
}

  AddConf $fid config(ftp,anonymous) "" {
# 1 -> Do anonymous FTP
# 0 -> Log in to FTP with username and password.
}

  AddConf $fid config(ftp,useproxy) "" {
# 1 -> Use defined ftp proxy
# 0 -> Don't use proxy (to access hosts inside the proxy/firewall)
}

  AddConf $fid config(ftp,cache,maxentries) "" {
# Maximum number of directory listings the cache should hold.
}

  AddConf $fid config(ftp,cd_pwd) "" {
# If true (non-zero): When doing a cd to a new directory, Filerunner
# will run pwd to check where it ended up (which makes a difference
# if you cd to a link which points to a directory).
}

  puts $fid {
#****** HTTP CONFIGURATION *********************************************
}

  AddConf $fid config(http,proxy) "" {
# HTTP proxy. Format: host:port (for example: proxy.foo.com:80). 
# Leave empty if you have a direct connection to the net.
}

  puts $fid {
#****** APPEARANCE CONFIGURATION ***************************************
}

  AddConf $fid config(gray) "" {
# The color gray is spelled grey in some places.  If you put your favorite
# spelling here, we promise to use that spelling when offering up color
# names that contain gray/grey.  Note that all named color values that have 
# gray in their names are duplicates of those with grey in their names (and
# vice versa) so we need contend with only one spelling.
}

  AddConf $fid config(gui,color_bg) "" {
# Background color of listboxes etc.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_fg) "" {
# Foreground color of listboxes etc.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_cmd) "" {
# Color for commands in the command window.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_select_bg) "" {
# Selection background color.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_select_fg) "" {
# Selection foreground color.
# You can edit this easier from the configuration menu.
}
  AddConf $fid config(gui,color_highlight_bg) "" {
# Highlight background color. This is the color of the last
# selection and is the second argument for the diff command.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_highlight_fg) "" {
# Highlight foreground color.This is the color of the last
# selection and is the second argument for the diff command.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_handle) "" {
# The 'handle' color is the color of the 'handles' that allow
# you to change the size of the listbox columns.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_scheme) "" {
# Scheme color.
# Default Tk seems to be "grey85".
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_cursor) "" {
# Cursor color.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_flash) "" {
# Flash color is used in command completion messages as 
# well as in search messages in view windows and the 
# command windows.

# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,color_balloonHelp_fg) "" {
# Balloon Help foreground color.
# You can edit this easier from the configuration menu.
}
  AddConf $fid config(gui,color_balloonHelp_bg) "" {
# Balloon Help background color.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(ListBoxColumns,right) "2" {
# This structure defines the listbox layout.  The three parts are:
# The window name (never never NEVER change this),
# the window width,
# and the window label.
# You may change the label and the width, but the width is easier to 
# change by dragging the label separator in the window.
#
# Also, the order is the order the sub windows appear.  This you may change,
# but, again, it is easier to drag the label in the window to do the same thing.
#
# Note: the '[_  text ]' below tells the localize routine to replace the
# 'text' with a local version.  While this should be this way for the disto,
# feel free to eliminate the [_' and the ']'.
#
# You can add and delete entries form the list, however, the first part
# (the window name) must come from the 'config(ListBoxScripts)' (see below
# two entries down), so if you add and new script, you can then refer to
# it in this list.  Also, a given 'window name' may only be used once in
# each of the list boxes. 
}
  AddConf $fid config(ListBoxColumns,left) "2" {
# This is the same as above but for the left listbox.  Usually they will be
# the same, but that is not required.
}
  AddConf $fid config(ListBoxScripts) "2" {
# This is list of scripts that filerunner will use to format column entries
# in the directory list boxes.  This is here to allow you to add new columns.
# Each column window is filled with the info provided by the script.  The
# following variables are available for this script:
# $file   file name (without the directory part)
# $ffile  file name with the '@' and '/' tags attached
# $dir    directory name
# $type   file type (n normal, d directory, l normal link, ld directory link)
#         FTP file types are the same as above with a leading 'f' 
# $size   file size
# $mtime, $ctime, $atime The modify, create and access times for the file
# $mode   file mode in wrx format
# $usergroup  files owner ship
# $link   link name or blank
# $nlink  number of links in the files dir entry
# $inst   either 'left' or 'right', indicates the file list pane

# Please note that these entries are live.  If you mess them up it is possible
# or even likely that filerunner will not run.  Do keep a back up config file
# if you plan to tread here.  Also note that the entry 'short' (if below) will
# allow windows users to add a 'short name' column to the list.
  }

  AddConf $fid config(columnScroll) "" {
# This controls the position of the column scroll bar.  Possible values are:
# 0 No column scroll bar. 
# 1 Top of the column
# 3 Bottom of the column
#  You can also pull the windows left & right with the mouse.
  }
  AddConf $fid config(columnScrollSize) "" {
# This sets the thickness of the column scroll bar.  Recommended size is 8.
# The other scroll bars are sized at 11.  This size affects the minimum 
# column width (must be wide enough to contain the two end arrows).
  }

  AddConf $fid config(gui,ListBoxFont) "" {
# Font for listboxes etc. Must be fixed-width.
# A good choice is "TkFixedFont"
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,GuiFont) "" {
# Font for default Tcl/Tk stuff (menus, buttons etc.)
# Set this to "TkDefaultFont" in order to not change from what it already is.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(gui,BalloonHelpFont) "" {
# Font for Balloon Help text
# Set this to "TkTooltipFont" in order to not change from what it already is.
# You can edit this easier from the configuration menu.
}

  AddConf $fid config(geometry,main) "" {
# The X11 geometry string deciding where and how big the Filerunner
# main window should be at start up.
}

  AddConf $fid config(geometry,textviewer) "" {
# The X11 geometry string deciding where and how big the Filerunner
# text viewer window should be.
}

  AddConf $fid config(geometry,qedit) "" {
# The X11 geometry string deciding where and how big the Filerunner
# quick-editor window should be.
}

  AddConf $fid config(middle_button_list) "2" {
# This is a list of the middle buttons in the order they will be
# presented.  User defined buttons will be added automatically at the
# end if they are not already in the list. A button may be disabled by 
# appending "d" to the button or, if not a user defined button, deleting
# it or misspelling it.  (Misspelled buttons are quite not displayed.)
# Feel free to reorder this list.  Syntax is:
#    {ButtonName} or ButtonName if on a separate line and no spaces
# or {ButtonName d} for a disabled button
# or {{Button Name} d} if there are spaces in the name
# You need to restart Filerunner for this to have effect.
}

  AddConf $fid config(middle_button_colors) "" {
# This contains a list of background colors for the middle row command
# buttons. The syntax is:
#  set config(middle_button_colors) {
#    { ButtonName [-]Color }
#    ...
#  }
# An example:
#  set config(middle_button_colors) {
#    { Copy #0000ff }
#    { Delete yellow }
#    { Delete -red }
#    { Move -red }
#  }
# The Copy button inactive color will be blue and the Delete button 
# will be yellow. The Move and Delete buttons will turn red only when they
# are active i.e. when the mouse moves over them. Buttons not mentioned
# will get the default scheme color.
#
# Note: Since choosing an inactive color will automatic choose a lighter 
#       'active' color, if you want to choose both an inactive and an active 
#       color for the same button, you should arrange this list to put the
#       'active' color for the same button after the inactive color for it.
}

  puts $fid {
#****** SHELL CONFIGURATION ********************************************
}

  AddConf $fid config(shell,height,left) "" {
# Height in lines of text of the left shell text window.
}

  AddConf $fid config(shell,height,right) "" {
# Height in lines of text of the right shell text window.
}

  AddConf $fid config(shell,buffer) "" {
# Size of shell buffer, in lines of text.
}

  AddConf $fid config(shell,aliases) "{" {
# Shell aliases, in the form of a list of doublets: " { alias, command } .. "
# If there are arguments to the command line they are just tagged onto the
# alias. (I.e, aliases can take arguments)
}
  foreach k $config(shell,aliases) {
    if { [lindex $k 1] != "" } {
      puts $fid "    {$k}"
    } else {
      puts $fid "    $k"
    }
  }
  puts $fid "  }\n"

  puts $fid {
#****** MISC CONFIGURATION *********************************************
}

  AddConf $fid config(startpwd,left) "" {
# At start up, show this directory in the left panel.
# If you set it to "", Filerunner will show the directory from where 
# it was invoked.
}

  AddConf $fid config(startpwd,right) "" {
# At start up, show this directory in the right panel
# If you set it to "", Filerunner will show the directory from where 
# it was invoked.
}

  AddConf $fid config(fileshow,all) "" {
# 1 -> Will show hidden files (in *nix files beginning with a dot ".").
# 0 -> Won't.
}

  AddConf $fid config(fast_checkboxes) 2 {
# You can configure check boxes to appear on the top menu bar. The following
# are available.  You may sort these to get them to appear in your desired 
# order.  Eliminate the "d" to get the box to appear.  The second item
# in each entry is the value to use as a label for the check box.  Feel
# free to edit the label or even eliminate it (by using {}).  (Balloon help
# will still tell you what the box does.)
# You'll have to restart Filerunner to get this setting to have effect.
}

  AddConf $fid config(balloonhelp) "" {
# 1 -> Will enable balloon help
# 0 -> disables balloon help.
}

  AddConf $fid config(positiondirs) "" {
# 1 -> Will 'show' directories when typing a character in to a selected 
#      directory panel.
# 0 -> Will 'show' files when typing a character in to a selected 
#      directory panel.
# By 'show' is meant that the panel is scrolled so that files (or directories)
# that start with the typed character will visible.
}

  AddConf $fid config(focusFollowsMouse) "" {
# 1 -> Will enable focus following mouse
# 0 -> disables focus following mouse (requires restart of filerunner).
}

  AddConf $fid config(fileshow,dirs) "" {
# This variable determines how file listings are sorted. Possible values
# are  "dirsfirst", "dirslast", "mixed"  This is most easily changed with the 
# configuration menu.

}

  AddConf $fid config(fileshow,sort) "" {
# This variable determines what file listings are sorted on. Possible values
# are "nameonly", "time", "rtime", "size" and "extension"  This is most easily
# changed with the configuration menu.

}

AddConf $fid config(sortoption) "" {
# This variable determines how file listings are sorted. Possible values
# are "-ascii", "-nocase" and "-dictionary".  This is most easily changed 
# with the configuration menu. Ascii and nocase should be clear.  Dictionary
# extends "nocase" such that if two names contain embedded numbers the numbers
#compare as integers, not characters.   For  example, in dictionary mode, 
# bigBoy sorts between bigbang and bigboy, and x10y sorts between x9y and x11y.

}
  
  AddConf $fid config(dateformat) "" {
# Sets date format. Possible values "yymmdd" and "ddmmyy".
# (yy=year mm=month dd=day)
}

  AddConf $fid config(autoupdate) "" {
# Sets how often Filerunner checks to see if it needs to update its 
# windows.
# Set this to 0 to disable. Value is in seconds.
}

  AddConf $fid config(inotify_rate_limit) "" {
# Sets minimum amount of time between window updates when using inotify
# (see below).  This allows a trade off of prompt updates vs system load.
# Value is in seconds.
  }

  AddConf $fid config(inotify_flags) "" {
# inotify is only available in Linux systems.  These flags control what
# events will cause filerunner to update its directory listing(s).  This 
# update is nearly immediate, however, this requires 'some' overhead.  You
# can remove some or all of these flags to reduce the overhead. There are
# two sets of flags, the local ones (inotify_flags) and the non-local ones
# (inotify_nlflags).  Due to network overhead it is recommended that you
# use a more restricted (even to nill, i.e. {}) for non-local file systems.
# This is particularly true for the 'r', 'w', 'a' and 'o' flags.
# The flags have the following meanings:
#     n  File was created.
#     r  File was read from.
#     w  File was written to.
#     a  File's meta-data (inode or xattr) was changed.
#     C  File was closed (and was open for writing).
#     c  File was closed (and was not open for writing).
#     o  File was opened.
#     m  File was moved away from watch.
#     M  File was moved to watch.
#     d  File was deleted.
# 'r','w','a','C','c' and 'o' cause just the files line in the window
#  to be updated while the others require a full reread and display of 
#  the directory.
  }
  AddConf $fid config(inotify_nlflags) "" {
# The inotify_nlflags are the same as above but are used on directories
# that are not local (nfs or cifs).  Due to network overhead you will
# usually not want to be as aggressive with the Non Local flags.
  }
  AddConf $fid config(logsize) "" { 
# Size of log in bytes. Don't set this too high, or Filerunner might 
# slow down a little. When the log exceeds this size, it will be 
# truncated.  
}

  AddConf $fid config(check_ownership) "" {
# If this is "1", Filerunner will check as best as it can if you are
# about to overwrite or delete a file you do not own and warn you about
# it. This can be handy when working in groups with other people and you
# want an extra check that you don't accidentally erase someone else's
# files.  This will slow some operations down slightly.
}

  AddConf $fid config(keyb_support) "" {
# If this is 1, you will make the file listbox take focus when clicked
# and this will let you use the keyboard for traversing the list (using
# arrows and PgUp/PgDown) and issuing commands.
# You'll have to restart Filerunner to get this setting to have effect.
}

  AddConf $fid config(ask,dir_delete) "" {
# If 1, Filerunner will ask you if you are sure when you delete 
# directories. If 0, it will go ahead and delete without asking.
}

  AddConf $fid config(ask,file_delete) "" {
# If 1, Filerunner will ask you if you are sure when you delete 
# files. If 0, it will go ahead and delete without asking.
}

  AddConf $fid config(mwheel,delta) "" {
# For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
# number of lines to scroll when turning mouse wheel.
}

  AddConf $fid config(mwheel,neg) "" {
# For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
# mouse button to use for negative scroll movement.
}

  AddConf $fid config(mwheel,pos) "" {
# For "wheel"-mice (MS Intellimouse or Logitech MouseMan+), set
# mouse button to use for positive scroll movement.
}

  AddConf $fid config(save_conf_at_exit) "" {
# If true (non-zero): Filerunner will automatically save configuration and
# hotlist at exit.
}

  AddConf $fid config(create_relative_links) "" {
# If true (non-zero): Filerunner will create relative links
# instead of absolute links. (e.g ../../foo instead of /a/b/c/foo)
# Windows (NTFS) only allows absolute links, thus absolute is forced
# on Windows platforms.
}

  AddConf $fid config(cd_pwd) "" {
# If true (non-zero): When doing a cd to a new directory, Filerunner
# will run pwd to check where it ended up (which makes a difference
# if you cd to a link which points to a directory).
}

  AddConf $fid config(spellingFilter) "2" {
# The spelling filter chooses what part of a line is spell checked when
# you spell check in "View" "Q-Edit" windows.  In the entries below the
# filter name is first and is followed by the filter script.  You may add
# your own filter to this list (see the "Users Guide" for help on this).
# Several filters are provided and may be configured here. 
# Provided filters:
#
# {}        This is the default filter and causes the whole line to be checked.
#           This is the {None {}} entry below.
#
# qstrings  This filter can be configured to look for two types of strings:
#           Single lines (such as "# line" or "// line") as some languages use.
#           And strings that have start and end quote marks which may be
#           1 to N lines long.  E.g. "string" and "/* string */"
#
#           qstrings takes two parameters (as well as the string and line 
#           number that the spell check code supplies):
#           First, the beginning and ending quote strings, 
#                  e.g. {\" \"} or {/* */}
#           Second, the single line start quote string, e.g. {#} or {//}
#
# thisstr   This filter takes, as a parameter, another filter and selects
#           one of the the strings as the string to check.  This might be used
#           if you want to spell check a translation list where the 
#           original and the translation are two stings on the same line
#           and you want to only check the first or second string.  
#           For example {thisstr 1 {qstrings {\" \"} {}}} would check the
#           second string (use 0 for the first string).
  }

  puts $fid {
#****** DANGEROUS CONFIGURATION ****************************************
# Below this point you can change some of the stuff that is used
# internally by Filerunner. This should almost never be necessary if the
# platform is standard Unix or close to standard Unix. If you change any
# of this below, make very very sure you get it right. (Go read the
# source!). If you are unsure, don't touch it. If you get it wrong, very
# bad things can happen...
# 
# A note on having the full path to the commands below, like using
# "/bin/cp" and not just "cp": Lets say you have a dot "." in your path
# and someone really mean has created a command called "cp" in his home
# directory that does a "rm" instead. Then you cd to his home directory
# and type cp <something>, you will invoke his evil manipulated version
# of cp that removes whatever you intended to copy. The morale of this is
# to always use full path to critical commands and hope your sysop is
# not so mean he will rename "/bin/rm" to "/bin/mkdir"...
# A few commands below do not have specified full path, that's because
# they aren't very critical and seem to move around between different
# versions of Unix.
}
  AddConf $fid config(cmd,sh) "" {
# The bourne-shell command. This should almost always be /bin/sh.
# If an option (such as '-c') is needed to execute a command in the
# command line, it should be here (for example '/bin/sh -c').
# On Windows with out CYGWIN you can use 'cmd /C' (which is the default).
}

  AddConf $fid config(cmd,cp) "" {
# The cp (copy-files) command. You use the internal Tcl file copy routine 
# by making this empty ({}). Otherwise this should almost always be /bin/cp
# on Unix/Linux systems.
# The 'pfr' switches should be here (i.e. 'cp -pfr')
# (p->preserve times/owner)
# (f-> force overwrite)
# (r-> if dir, copy recursively)
# On Windows 'xcopy' works fine. Default is with switches: /I /Q /H /C /K /Y
# (/Q -> Don't display file names)
# (/H -> Copies hidden files)
# (/C -> Continue copying if errors)
# (/K -> Copy attributes)
# (/Y -> Suppresses prompting to confirm...)
#
# DON'T put the -i switch to cp here to get Filerunner to display
# a warning for overwriting a file when copying a file. It won't
# work and will only make it behave unpredictably.
}

  AddConf $fid config(cmd,dircp) "" {
# This command is used by the 'copy' button when the source is a directory.
# Again '{}' will use the Tcl file copy code and disallows back-grounding.
# On Unix/Linux systems this can be the same as the 'cp' command above
# However on Windows the 'xcopy' command will not copy a single file to
# a point deeper in the tree if the /S or /E switches are on and we do
# want one of those on in this command.
# (/S -> copies directories and sub-directories except empty dirs)
# (/E -> copies directories and sub-directories INCLUDING empty dirs) 
#
}

  AddConf $fid config(cmd,hardlink) "" {
# This command is used by the 'Hardlnk' button.  
# On unix/linux systems and Windows with CYGWIN the best option is:
# '{{cp -pfrl}}' 
# (for meaning of 'pfr' see comments on the 'cp' command above.)
# On windows systems without CYGWIN you can use 'fsutil hardlink create' 
# however, it appears that this command will not re-curse over directories.
# Also, this command wants the parameters in the reverse order which is
# indicated thusly: {{fsutil hardlink create} {r}}
}

  AddConf $fid config(cmd,chmod) "" {
# The change mode (chmod) command. This should almost always be /bin/chmod.
}
 AddConf $fid config(cmd,chown) "" {
# The change mode (chmod) command. This should almost always be /bin/chown.
}

  AddConf $fid config(cmd,touch) "" {
# The touch command.
}

  AddConf $fid config(cmd,du) "" {
# The "du" command. Should return size in kilobytes. On many
# systems other than BSD (?) you need to add a -k argument
# to this command to get it in kilobytes.
}

  AddConf $fid config(cmd,find) "" {
# The find command.
}

  close $fid
  Log "Configuration saved to $glob(conf_dir)/config$glob(cygwin)-$newv"
}

proc InitConfig {} {
#  puts "in InitConfig"
  global glob config env argv tcl_platform

  if { $glob(os) == "WIN32" } {
    set config(hotlist) [list [concat -volume [file volume ]]]
  } else {
    set config(hotlist) ""
  }
  set config(runlist) {}
  set config(balloonhelp) 1
  set config(fileshow,all) 0
  set config(focusFollowsMouse) 1
  set config(fileshow,sort) "nameonly"
  set config(fileshow,dirs) "dirsfirst"
  set config(columnScroll) 1
  set config(columnScrollSize) 8
  set config(positiondirs) 0

  set config(startpwd,left) ""
  set config(startpwd,right) ""
  set config(ask,dir_delete) "1"
  set config(ask,file_delete) "0"
  set config(ftp,fastlink) 0
  set config(ftp,proxy) ""
  set config(ftp,user) anonymous
  set config(ftp,password) ""
  set config(ftp,timeout) "120"
  set config(ftp,anonymous) 1
  set config(ftp,useproxy) 1
  set config(ftp,login) {}
  set config(ftp,cache,maxentries) 15
  set config(ftp,cd_pwd) 1
  set config(http,proxy) {}
  #set config(gui,color_bg) FloralWhite
  set config(gray) "gray"
  set config(gui,color_bg) LemonChiffon
  set config(gui,color_fg) Black
  set config(gui,color_cmd) LightGoldenrod1
  set config(gui,color_select_bg) Yellow
  set config(gui,color_select_fg) black
  set config(gui,color_highlight_bg) white
  set config(gui,color_highlight_fg) red
  set config(gui,color_handle) red  
  set config(gui,color_scheme) "grey85"
  #set config(gui,color_scheme) {#bfeed0add9da}
  set config(gui,color_cursor) black
  set config(gui,color_flash) green
  set config(gui,color_balloonHelp_fg) yellow
  set config(gui,color_balloonHelp_bg) red
  set config(gui,GuiFont) TkDefaultFont

  set config(gui,ListBoxFont) TkFixedFont
  set config(gui,BalloonHelpFont) TkCaptionFont
  
  set config(ListBoxColumns,right) {
    {file 26   {[_ "File/Dir name" ]}}
    {links  3     {[_ "lk" ]}}
    {size 7    {[_ "size" ]}}
    {mtime 15  {[_ "mtime" ]}}
    {mode 9   {[_ "flags" ]}}
    {owner 13  {[_ "owner" ]}}
    {slink  26 {[_ "link to" ]}}
  }
  set config(ListBoxColumns,left) {
    {file 26   {[_ "File/Dir name" ]}}
    {links  3     {[_ "lk" ]}}
    {size 7    {[_ "size" ]}}
    {mtime 15  {[_ "mtime" ]}}
    {mode 9   {[_ "flags" ]}}
    {owner 13  {[_ "owner" ]}}
    {slink  26 {[_ "link to" ]}}
  }

  set config(middle_button_colors) {
    { Delete -#ff4d4d }
  }
  set config(middle_button_list) {
    Copy 
    CopyAs
    Delete
    Move
    Rename
    MkDir
    S-Link
    S-LnAs
    Chmod
    View
    Edit 
    Q-Edit
    Arc
    UnArc
    UnPack
    ForEach
    Print
    Diff
    Select
    HardLnk
    Open
    Run
  }
  set config(fast_checkboxes) {
    {{Expand Errors} {Er Exp} d}
    {{Balloon Help} BHelp }
    {{Position to: directories} {P to D}}
    {{Show All Files} All d}
    {{Create Relative Links} RL d}
    {{Run Pwd After Cd} Pwd d}
    {{Run Pwd After Cd (FTP)} PwdFTP d}
    {{Focus Follows Mouse} FM d}
    {{Anonynomous Ftp} AnFTP d}
    {{Use FTP Proxy} Prox d}
  }

  set config(bind,Button-3) ViewOne
  set config(bind,Double-1) {DoMenu,three}
  set config(bind,Control-3) ViewDirOpposite
  set config(bind,Control-1) {DoMenu,sample}
  set config(bind,Button-2) {DoMenu,two}
  set config(global-bind,Button-3) {DoMenu,two}
  #
  # A menu to use with one of the bindings (user can build more)
  #
  set config(menu,sample) [list ViewOne Edit {} menu,two ViewDirOpposite]
  set config(menu,two) [concat {ViewOne} $config(middle_button_list)]
  set config(menu,three) {{{Send To} {CmdRunCmd 0}}}

  set config(no_selection) {ViewOne ViewDirOpposite}

  # These are, at the moment, mutually exclusive...
  #      Sort On Mixed
  #      Sort On Dirs First
  #      Sort On Dirs Last
  # and then
  #        Sort On Time
  #        Sort On Reverse Time
  #        Sort On Size
  #        Sort On Extension
  #        Sort On Name
  set config(paste) {{<Button2-ButtonRelease>}}
  set config(space) {>sp<}
  set config(keyb_support) 0
  set config(dateformat) "yymmdd"
  set config(autoupdate) 20
  set config(inotify_rate_limit) 2
  set config(inotify_flags) "naCmMdw"
  set config(inotify_nlflags) ""
  set config(logsize) 5000
  set config(check_ownership) 0
  set config(geometry,main) "840x720+31+0"
  set config(geometry,textviewer) "80x30"
  set config(geometry,qedit) "80x30"
  set config(spellingFilter) {
    {None {}}
    {Comment(#&\") {qstrings {\" \"} "#"}}
    {C&C++Comments {qstrings {/* */} "//"}}
  }
  set config(shell,height,left) 10
  set config(shell,height,right) 10
  set config(shell,buffer) 1000
  set config(shell,aliases) {
    {.. {cd ..}} 
    {md mkdir} 
    {m view} 
    {more view} 
    {less view}
  }
  #
  # The helper list...
  set postfix [expr {$glob(cygwin) != {} ? ".exe" : ""}]
  set term xterm$postfix
  set editor "$term$postfix -e vi %s"
  set print "lpr$postfix %s"
  set diff "diff$postfix -r %s %s"
  set pack "gzip$postfix %s"
  set archive "tar+gz %s"

  set config(cmd,sh) "sh -c"
  set config(cmd,cp) "cp -pfr"
  set config(cmd,dircp) "cp -pfr"
  set config(cmd,hardlink) "cp -pfrl"
  set config(cmd,open) "xdg-open"
  set config(editor) "vi %s"

  if { $glob(os) == "WIN32" } {
    set config(cmd,open) {cmd /c}
    set config(editor) "{C:/Program Files/Windows NT/Accessories/wordpad.exe} %s"
    if {[info exists env(CYGWIN)] == 0 } {
      # pure windows...
      set config(cmd,sh) "cmd /C"
      set config(cmd,cp) "xcopy /I /Q /H /C /K /Y"
      set config(cmd,dircp) "xcopy /I /Q /H /C /K /Y /E"
      set config(cmd,hardlink) {{fsutil hardlink create} {r}}
    } else {
      # windows with cygwin (NOT a cygwin version of tcl/tk but rather
      # cygwin is installed...
      # Use the non-windows defs...
    }
  }
  if {[info exists env(EDITOR)]} {
    set config(editor) "$env(EDITOR) %s"
  }
  set config(cmd,chmod) "/bin/chmod$postfix"
  set config(cmd,chown) "/bin/chown$postfix"
  set config(cmd,touch) "touch$postfix"
  set config(cmd,du) "du$postfix -s"
  set config(cmd,find) find$postfix
  set config(cmd,print) "lpr$postfix %s"
  set config(cmd,term) "xterm$postfix"
  set config(cmd,diff) "diff$postfix -r %s %s"
  set config(cmd,pack) "gzip$postfix %s"
  set config(cmd,archive) "tar+gz %s"
  
  set config(cmd,unarc,extensions) \
      [list \
	   [list  "gzip$postfix -cd %s | tar xfBp -" \
		{*.tgz *.taz *.tar.z *.tar.gz} ] \
	   [list "tar$postfix xfBp %s"\
		{*.tar} ]\
	   [list "unzip -oqq %s" \
		{*.zip} ]\
	   [list "lha$postfix -x %s" \
		{*.lha} ]\
	  ]
  
  set config(cmd,unpack,extensions) \
      [list \
	   [list "gzip$postfix -d %s" \
		{*.gz} ]\
	  ]
  set config(view,extensions) \
      [list \
	   [list "xmikmod$postfix %s" \
		{*.mod.gz *.s3m *.mtm *.stm *.xm *.uni *.mod} ]\
	   [list "timidity$postfix %s" \
		{*.midi.gz *.midi *.mid.gz *.mid} ]\
	   [list "xv$postfix -gamma 2.0 %s" \
		{ *.bmp *.iff *.jpg *.pbm *.pgm *.png \
		      *.ppm *.tga *.tif *.tiff *.xbm *.xpm *.gif} ]\
	   [list "gzip$postfix -dc %s | ghostview -" \
		{ *.ps.gz } ]\
	   [list "ghostview$postfix %s" \
		{ *.ps } ]\
	   [list "xanim$postfix %s" \
		{*.avi *.fli *.flc *.mpg} ]\
	   [list "gzip$postfix -cd %s | tar tvf -" \
		{ *.tgz *.taz *.tar.z *.tar.gz } \
		-viewtext ]\
	   [list "tar$postfix tvf %s" \
		{*.tar}\
		-viewtext ]\
	   [list "unzip$postfix -lv %s"\
		{*.zip}\
		-viewtext]\
	   [list "lha$postfix -v %s" \
		{*.lha}\
		-viewtext ]\
	   [list "gzip$postfix -cd %s"\
		{*.gz}\
		-viewtext ]\
	   [list "rpm$postfix -qilp %s"\
		{*.rpm}\
		-viewtext ]\
	   ]
  set config(open,extensions) {}
  set config(mwheel,delta) 6
  set config(mwheel,neg) <4>
  set config(mwheel,pos) <5>
  set config(save_conf_at_exit) 1
  
  if {$tcl_platform(platform) != "windows" } {
    set config(create_relative_links) 1
  } else {
    set config(create_relative_links) 0
  }
  
  set config(cd_pwd) 1
  set config(sortoption) "-ascii"
  # Listbox column build scritps.  There should be one for each list box
  # column, by name.
  set config(ListBoxScripts) { 
    {ctime {expr {$ctime=="" ? "" : [GetTimeFromSecs $ctime]}}}
    {atime {expr {$atime=="" ? "" : [GetTimeFromSecs $atime]}}}
    {mtime {GetTimeFromSecs $mtime}}
    {file  {expr {$ffile}}}
    {mode  {expr {$mode}}}
    {owner {expr {$usergroup}}}
    {links {expr {$nlink}}}
    {size  {format %6s $size}}
    {slink {format %3s $link}}
    {blank {expr {""}}}
  }

  if {$glob(os) == "WIN32" } {
    lappend config(ListBoxScripts) {short {file attributes $file -shortname}}
  }
  set glob(df) df
  set glob(searchstring) ""
  set glob(log) ""
  set glob(left,pwd)  [frPwd]
  set glob(right,pwd) [frPwd]
  set glob(left,update_oldpwd) $glob(left,pwd)
  set glob(right,update_oldpwd) $glob(right,pwd)
  set glob(left,newpwd_oldpwd) $glob(left,pwd)
  set glob(right,newpwd_oldpwd) $glob(right,pwd)
  set glob(left,filelist) {}
  set glob(right,filelist) {}

  set glob(hotlist) {}
  set glob(toplevelidx) 0
  set glob(entry_dialog_return) {}
  set glob(ftp_entry_dialog_return) {}
  set glob(ftp,serverport,start) 5000
  set glob(ftp,serverport,end) 65000
  # This is an ugly hack to avoid reusing ports too fast, 
  # but it's because of a limitiation in the tcl socket
  # command. (You can't make the socket -server command 
  # choose a port by itself...)
  set glob(ftp,serverport,last) [expr 5000 + (([pid] * 100) % (65000 - 5000))]
  set glob(ftp,serversocket,acceptlist) {}
  set glob(forceupdate) 0
  set glob(havedoneftp) 0
  set glob(enableautoupdate) 1
  set glob(left,df) ?
  set glob(right,df) ?
  set glob(abortcmd) 0
  set glob(left,dirstack) ""
  set glob(right,dirstack) ""
  set glob(left,shell,packed) 0
  set glob(right,shell,packed) 0
  set glob(shell,maxed) 0
  set glob(left,shell,history) {}
  set glob(right,shell,history) {}
  set glob(left,shell,complete,flipping) 0
  set glob(right,shell,complete,flipping) 0
  set glob(left,shell,history,flipping) 0
  set glob(right,shell,history,flipping) 0
  set glob(left,shell,history,flipping,index) 0
  set glob(right,shell,history,flipping,index) 0
  set glob(batchlist) {}
  set glob(tmpdir) $glob(conf_dir)/tmp

  set glob(win,top) .fupper.ftop
  set glob(win,left) .fupper.fleft
  set glob(win,right) .fupper.fright
  set glob(win,middle) .fupper.can.fmiddle
  set glob(win,bottom) .flower

  set glob(gui,color_xx,winlist) ""
  set glob(gui,color_cmd,winlist) ""
  set glob(gui,color_scheme,winlist) ""
  set glob(gui,tablist) ""

  # These configuration have not been set yet, will be done in ReConfigColors
  set glob(gui,color_bg)  ""
  set glob(gui,color_fg)   ""
  set glob(gui,color_cmd)  ""
  set glob(gui,color_select_bg) ""
  set glob(gui,color_select_fg)  ""
  set glob(gui,color_highlight_bg) ""
  set glob(gui,color_highlight_fg) ""
  set glob(gui,color_balloonHelp_fg) ""
  set glob(gui,color_balloonHelp_bg) ""
  set glob(gui,color_scheme) ""
  set glob(gui,GuiFont) ""
  set glob(gui,color_cursor) ""
  set glob(gui,color_handle) ""

  set glob(gui,ListBoxFont) $config(gui,ListBoxFont)
  set glob(gui,GuiFont) ""
  set glob(gui,BalloonHelpFont)  $config(gui,BalloonHelpFont)  
  set glob(async) 0
  set glob(left,shellcount) 0
  set glob(right,shellcount) 0

  foreach ev {USER USERNAME LOGNAME} {
    if {![info exists env($ev)]} { 
      continue 
    }
    if {"[string range $env($ev) 0 0][string range $env($ev) end end]" == \
	    "%%" } {
      if {[string equal -nocase $ev "USER" ]} {
	unset env(USER)
      }
      continue
    }
    set env(USER) $env($ev)
    break
  }
  if {![info exists env(USER)]} { 
    if {![info exists env(HOME)]} {
      PopInfo [_ "Cannot read environment variables\
                  USER or USERNAME or LOGNAME or HOME.\
                  \nFileRunner cannot work without HOME and \
                  \n one of USER or USERNAME or LOGNAME correctly set."]
      CleanUp 1
    } else {
      set env(USER) [file tail $env(HOME)]
      PopInfo [_ "Environment variables 'USER' ane 'USERNAME' are undefined.\
                 \nDefaulting to $env(USER).\
                 \nTo stop this error report each time you\
                  run filerunner define environment variable\
                  'USER' or 'USERNAME'."]
    }
  }
  
  if {![info exists env(HOST)]} {
    if {![info exist env(COMPUTERNAME)] } {
      set env(HOST) unknown
    } else {
      set env(HOST) $env(COMPUTERNAME)
    }
  }
  if {$env(HOST) == "unknown"} {
    set r [catch { exec uname -n } out]
    if {!$r} {
      set env(HOST) $out
    }
  }

  set glob(history) ""
  # read history from disk
  if { [file exists $glob(conf_dir)/history ] } {
    set r [catch {set fid [open $glob(conf_dir)/history r]
      gets $fid glob(history)
      close $fid} out]
    if {$r} {
      puts [_ "FileRunner: Can't read directory history from disk: $out"]
    }
  }

  # set up a few default options
  #  option add *HighlightThickness 1 widgetDefault
  option add *InsertWidth 2 widgetDefault
  option add *Scrollbar.Width 11 widgetDefault
  option add *Button.BorderWidth 1 widgetDefault
  option add *Scrollbar.BorderWidth 2 widgetDefault
  #  option add *Listbox.BorderWidth 2 widgetDefault 
  # makes the spacing between list elements too large...
  option add *Frame.BorderWidth 2 widgetDefault
  option add *Entry.BorderWidth 2 widgetDefault
  #  option add *.Text.InsertBackground red widgetDefault
  #  option add *insertBorderWidth 2 widgetDefault
  #  option add *BorderWidth 1 widgetDefault
}

proc ReadConfig {{re 0}} {
  global glob glob config env argv argv0 tcl_platform
  set glob(select_pry_lr) {}
  # This gets rid of old menus and bindings.  
  #   array unset config menu,*
  #   array unset config bind,*
  lassign [getOldNewVersions] oldv newv
  set desired $glob(conf_dir)/config$glob(cygwin)-$newv
  if {0} {
  if {$newv < $oldv} {
    set first $glob(conf_dir)/config$glob(cygwin)
    set sec  $glob(conf_dir)/config$glob(cygwin)-$oldv
  } else {
    set first $glob(conf_dir)/config$glob(cygwin)-$oldv
    set sec  $glob(conf_dir)/config$glob(cygwin)
  }}
  if {![file readable $desired] } {
    Log "Configuration file $desired not found/readable."
    set m \[0-9\]
    cd $glob(conf_dir)
    # This should find all our old (and newer) configuration files...
    set pos \
	[lsort -decreasing \
	     [glob -nocomplain -types {f r w} config-\[1-2\]$m$m$m$m$m config]]
    # on a new system or user there will be NO file...
    set from {}
    foreach from $pos {
      # here we insure that what we use is from an older rev and not a newer
      if {[string compare $glob(conf_dir)/$from $desired] == -1 } {
	set from $glob(conf_dir)/$from
	break
      }
    }
    if {$from == {}} {
      # if all that failed, check for the old name without the rev
      set from $glob(conf_dir)/config$glob(cygwin)
    }
    if {[file readable $from]} {
      Log "Found Configuration file $from. Moving to $desired"
      catch {file copy $from $desired}
    }
  }
  if {[file readable $desired]} {
    #     puts "sourcing $desired"
    Log "Reading Configuration from $desired"
    set r [catch {source $desired} out]
    if {$r} {
      smart_dialog .error_dialog "." [_ "Init error"] \
	  [list [_ "Reading user configuration\
           %s:\n%s" $desired  $out]] \
	  0 1 "OK" {maxw 16000}
    }
  }
  lappend ::startTimes [list [clock milliseconds] "After read config file"]
  if {$glob(cygwin) != {}} {
    # make sure df is properly named.
    set ops [FilenameComplete df 1]
    if {[lindex $ops 0] == {}} {
      # found df exact
      set glob(df) df
    } else {
      set idx [lsearch -glob $ops ".exe*"]
      if {$idx != -1} {
	set glob(df) df[lindex $ops $idx]
      } else {
	set glob(df) {}
      }
    }
  }
  # We are about to re do the dir listing windows
  # First we must prevent the inotify code from updating while
  # The window isn't
  foreach inst {left right} {
    if {[info exists glob(inotify_after,$inst)] && \
	    $glob(inotify_after,$inst) != {}} {
      catch {after cancel $glob(inotify_after,$inst)}
    }
    set glob(inotify_after,$inst) "no"
    ClearCherryPicker $inst
  }
  catch {destroy $glob(win,left).frame_listb}
  catch {destroy $glob(win,right).frame_listb}
  set glob(hotlist) $config(hotlist)
  if {$config(ftp,password) == "" } {
    set r [catch {set config(ftp,password) "$env(USER)@[exec domainname]"}]
  }
  # encript any open text passwords
  FixPasswords
  # build the listbox script array
  foreach val $config(ListBoxScripts) {
    lassign $val name script
    set glob(lbscript,$name) $script
  }
  foreach k { left right } {
    #    puts "$config(ListBoxColumns,$k)"
    foreach ent $config(ListBoxColumns,$k) {
      lappend glob(listboxNames) [lindex $ent 0]
    }
  }
  #  puts "$glob(listboxNames)"
  set glob(listboxNames) [lsort -unique $glob(listboxNames)]
  foreach inst {left right} {
    foreach lbname $glob(listboxNames) {
      set glob($inst,lv$lbname) {}
    }
    if { $config(startpwd,$k) != "" && !$glob(init_done)} {
      catch { cd $config(startpwd,$k); NewPwd $k [frPwd] } out
    }
  }
  if {$glob(notify,Available)} {
    set glob(inotify_flags) $config(inotify_flags)
  }
  if { $config(ftp,proxy) == "" } {
    set config(ftp,useproxy) 0
  }
  if { [info exists config(focusFollowsMouse)] == 1 } {
    if { $config(focusFollowsMouse) != 0 } {
      tk_focusFollowsMouse
    }
  }
  if {$tcl_platform(platform) == "windows" } {
    set config(create_relative_links) 0
  }
  if { $config(dateformat) == "ddmmyy" } {
    GetTimeFromSecsSetFormat 1
  } elseif { $config(dateformat) == "yymmdd" } {
    GetTimeFromSecsSetFormat 0
  } else {
    GetTimeFromSecsSetFormat  $config(dateformat)
  }
  set ::balloon_help::enable $config(balloonhelp)
  lappend ::startTimes [list [clock milliseconds] "After post config show"]
  postConfigShow
  BuildListBoxes
  # Really don't know why this needs to be called again, but if not
  # the center buttons aren't centered and the button scroll is off
  # especially with larger gui fonts
  postConfigShow
  lappend ::startTimes [list [clock milliseconds] "After post config list box build"]
  #  ReConfigColors
  #  ReConfigFont
  InitWindows
  lappend ::startTimes [list [clock milliseconds] "After post config init"]
  ForceUpdate
  # Allow the inotify code to do its thing again
  foreach inst {left right} {
    set glob(inotify_after,$inst) {}
  }
  lappend ::startTimes [list [clock milliseconds] "After post config update"]
  InitBindings
  if {$config(paste) != {}} {
    event add <<Paste>> {*}$config(paste)
  }
  # Here we do global bindings (config(global-bind,*))
  # First we deleate the old bindings...
  foreach {but val} [array get glob "bind,*,glob"] {
    set button [regsub {bind,(.*),glob} $but {\1}]
    catch {bind . <$button> {} }
  }
  # Now the new ones
  foreach {but val} [array get config "global-bind,*"] {
    set button [regsub {global-bind,(.*)} $but {\1}]
    catch {bind . <$button> {} }
    set glob(bind,$button,glob) [findCommand $val left]
    if {[catch {
      bind . <$button> "DoBut $button glob 0 %X %Y"} \
	     out] != 0 
      } {
      PopError [_ "In trying to do a global bind of '%s' the \
                \nerror '%s' occured. \
                 \n Skipping this binding." $button $out]
    }
  }

}

proc FixPasswords {} {
  global config env
  # encript any open text passwords
  set new {}
  foreach login $config(ftp,login) {
    if { [lindex $login 1 1 0] == ">>" &&  [lindex $login 1 1 end] == "<<" } {
      set passwd [lindex $login 1 1]
    } else {
      set passwd [concat ">>" [encript $env(USER) [lindex $login 1 1]] "<<" ]
    }
    set tmp [list [lindex $login 0] [list [lindex $login 1 0] $passwd]]
    if {[lindex $login 2] != "" } { lappend tmp [lindex $login 2]}    
    
    lappend new $tmp
  }
  set config(ftp,login) $new
}

proc ConfigPwd {} {
  global glob config env argv argv0
  set pwd(left) [lindex $argv 0]
  set pwd(right) [lindex $argv 1]
  foreach k { left right } {
    if {$pwd($k) != "" && $glob($k,pwd) != $pwd($k) } {
      NewPwd $k $pwd($k)
      UpdateWindow $k
    } else {
      if {$config(startpwd,$k) != "" && $glob($k,pwd) != $config(startpwd,$k)} {
        NewPwd $k $config(startpwd,$k) 
        UpdateWindow $k
      }
    }
  }
  AppendToDirHistory $glob(left,pwd)
  AppendToDirHistory $glob(right,pwd)
  set glob(left,dirstack) $glob(left,pwd)
  set glob(right,dirstack) $glob(right,pwd)
}

proc CheckConfigDir {} {
  global env glob

  if {[file isdirectory $glob(conf_dir)]} {
    return
  }
  # The wm command here moves the following question to the center 
  #(or there about) of the screen rather that having it get lost on an edge.
  wm geometry . +500+500
  if {$glob(os) != "WIN32" && [file exists $env(HOME)/.fr]} {
    return
  }
  set button [smart_dialog .questDialog "." [_ "Question"] [list [_ \
		  "The FileRunner configuration directory %s\
       does not exist. Ok to create it?" $glob(conf_dir)]] \
		  0 2 [list [_ "Yes"] [_ "No"]] {maxw 16000}] 
  if {$button == 1} {
    return
  }

  set r [catch [file mkdir $glob(conf_dir)] out]
  if {$r} {
    regsub -all {\\} $out {/} out
    PopErrorSimple "$out"
    exit 1
  }
}

proc ConfigBrowser {} {
  global glob config
  lassign [getOldNewVersions] oldv newv

  if {![file exists $glob(conf_dir)/config$glob(cygwin)-$newv]} { SaveConfig }

  set glob(edit_configuration_configlist) {
    {Hotlists {HOTLISTS}}
    hotlist
    runlist
    {{Mouse Bindings & menus} {MOUSE & KEY BINDINGS & MENUS}}
    {{  bind} {# set config(bind,Button-2)}}
    {{  menu} {Mail To}}
    {{  exception} no_selection}
    paste
    {Commands {COMMAND CONFIGURATION}}
    space
    editor
    view,extensions
    open,extensions
    cmd,open
    cmd,print
    cmd,term
    cmd,diff
    {Archiver/Packer {ARCHIVER/PACKER}}
    cmd,unarc,extensions
    cmd,unpack,extensions
    cmd,pack
    cmd,archive
    {FTP {FTP CONFIGURATION}}
    ftp,login
    ftp,fastlink
    ftp,proxy
    ftp,password
    ftp,timeout
    ftp,anonymous
    ftp,useproxy
    ftp,cache,maxentries
    ftp,cd_pwd
    {HTTP {HTTP}}
    http,proxy
    {Appearance {APPEARANCE}}
    gray
    gui,color_bg
    gui,color_fg
    gui,color_cmd
    gui,color_select_bg
    gui,color_select_fg
    gui,color_highlight_bg
    gui,color_highlight_fg
    gui,color_handle
    gui,color_scheme
    gui,color_cursor
    gui,color_flash
    gui,color_balloonHelp_fg
    gui,color_balloonHelp_bg
    gui,ListBoxFont
    gui,GuiFont
    gui,BalloonHelpFont
    columnScroll
    columnScrollSize
    ListBoxColumns,right
    ListBoxColumns,left
    ListBoxScripts
    geometry,main
    geometry,textviewer
    geometry,qedit
    middle_button_list
    middle_button_colors
    {Shell {SHELL}}
    shell,height,left
    shell,height,right
    shell,buffer
    shell,aliases
    {Miscellaneous {MISC}}
    startpwd,left
    startpwd,right
    balloonhelp
    focusFollowsMouse
    fileshow,all
    fileshow,dirs
    fileshow,sort
    fast_checkboxes
    dateformat
    autoupdate
    inotify_flags
    logsize
    check_ownership
    keyb_support
    ask,dir_delete
    ask,file_delete
    mwheel,delta
    mwheel,neg
    mwheel,pos
    save_conf_at_exit
    create_relative_links
    cd_pwd
    spellingFilter
    {{Dangerous stuff} {DANGEROUS}}
    cmd,sh
    cmd,cp
    cmd,dircp
    cmd,hardlink
    cmd,chmod
    cmd,touch
    cmd,du
    cmd,find
  }

  set w .config_dialog
  toplevel $w -class Dialog
  wm title $w "Configuration Browser"
  wm iconname $w "FR config browser"
  wm resizable $w true true
  wm transient $w [winfo toplevel [winfo parent $w]]

  frame $w.left
  frame $w.right
  frame $w.bot
  
  # Create left frame containing the list of config options
  scrollbar $w.left.scrollvert -command "$w.left.list yview" -orient vertical 
  listbox $w.left.list \
      -yscrollcommand "$w.left.scrollvert set" \
      -font $config(gui,ListBoxFont) \
      -width 25 \
      -height 35 
      # -background $config(gui,color_bg)\
      # -foreground $config(gui,color_fg) \
      # -selectbackground $config(gui,color_select_bg)\
      # -selectforeground $config(gui,color_select_fg) 
  pack $w.left.list -side left -fill both
  pack $w.left.scrollvert -side right -fill y

  # Create right frame containing the config text and a scrollbar
  text $w.right.text\
      -relief sunken \
      -bd 2 \
      -yscrollcommand "$w.right.scroll set" \
      -setgrid 1 \
      -height 35 \
      -undo 1 \
      -font $config(gui,ListBoxFont)\
      -highlightthickness 0 
      # -background $config(gui,color_bg) \
      # -foreground $config(gui,color_fg) \
      # -selectbackground $config(gui,color_select_bg) \
      # -selectforeground $config(gui,color_select_fg) 
  scrollbar $w.right.scroll -command "$w.right.text yview" 
  pack $w.right.scroll -side right -fill y
  pack $w.right.text -expand 1 -fill both

  # Create bottom frame containing the OK/Cancel buttons
  button $w.bot.regen \
      -text {Regenerate Configuration}  \
      -command "ConfigBrowserRegenerate $w.right.text"
  button $w.bot.ok -text OK -command "ConfigBrowserOK $w.right.text; destroy $w"
  button $w.bot.cancel \
      -text Cancel  \
      -command "set glob(config_dialog_return) {}; destroy $w"
  pack $w.bot.cancel -side right -padx 3 -fill x
  pack $w.bot.ok -side right -padx 3
  pack $w.bot.regen -side right -padx 3 -fill x

  # Fill left frame with contents
  #set glob(font_dialog,fl) [split $glob(font_dialog,fl) "\n"]
  set cdl {}
  foreach k $glob(edit_configuration_configlist) {
    if {[lindex $k 1] != {}} {
      lappend cdl [lindex $k 0]
    } else {
      set tmp [split $k ,_]
      set tmp2 {}
      foreach t $tmp {
        lappend tmp2 "[string toupper [string index $t 0]][string range $t 1 end]"
      }
      lappend cdl "  $tmp2"
    }
  }

  eval $w.left.list insert end $cdl

  # Create popup menu
  textSearch $w.right.text [_ "Cfg edit"] "+buildViewConfig" {} \
      [list Undo "undoHelp $w.right.text undo" \
	   Redo  "undoHelp $w.right.text redo" ]    
  
  pack $w.bot -side bottom
  pack $w.left -side left -fill both
  pack $w.right -side right -fill both -expand 1

  # Assign bindings
  bind $w.right.text $config(mwheel,neg) \
      "$w.right.text yview scroll -$config(mwheel,delta) units;break"
  bind $w.right.text $config(mwheel,pos) \
      "$w.right.text yview scroll $config(mwheel,delta) units;break"
  bind $w.left.list $config(mwheel,neg) \
      "$w.left.list yview scroll -$config(mwheel,delta) units;break"
  bind $w.left.list $config(mwheel,pos) \
      "$w.left.list yview scroll $config(mwheel,delta) units;break"
  bind $w.left.list <1> \
      "ConfigBrowserJumpToLoc $w.right.text \[$w.left.list nearest %y\]"
  wm withdraw $w
  update idletasks
  centerWin $w
  wm deiconify $w

  # Fill right frame with contents
  set filename $glob(conf_dir)/config$glob(cygwin)-$newv
  set r [catch {set fid [open $filename r]} out]
  if {$r} {
    PopError "Can't open $filename for editing"
    destroy $w
    return
  }
  $w.right.text insert 0.0 [read -nonewline $fid]
  $w.right.text edit reset
  close $fid
  $w.right.text mark set insert 0.0
}

proc ConfigBrowserJumpToLoc { w index } {
  global glob config
  set tmp [lindex $glob(edit_configuration_configlist) $index]
  if {[lindex $tmp 1] != {}} {
    set searchfor [lindex $tmp 1]
    set heading 1
  } else {
    set searchfor config($tmp)
    set heading 0
  }

  set tag select
  $w tag configure $tag\
      -background $config(gui,color_select_bg)\
      -foreground $config(gui,color_select_fg) 
  $w tag remove $tag 0.0 end

  set idx [$w search -count len -backwards -- $searchfor 0.0]
  if {$idx == ""} {
    PopWarn "Can't find that location in config file. \
      Try to regenerate configuration and try again."
    return
  }
  $w mark set insert "$idx + $len chars"
  $w see end
  $w see insert
  #  puts "$idx $len"
  set line [lindex [split $idx .] 0]
  if {$heading} {
    $w mark set insert $line.0
  } else {
    # search backwards for ------------ and make sure it's visible
    set idx [$w search -count len -backwards -- "-----------" insert]
    if {$idx != {}} {
      $w see $idx
      # The above may have pushed the object out.  Put it back.
      $w see insert
    }
  }
  $w tag add $tag $line.0 [expr $line+1].0
}

proc ConfigBrowserOK { w } {
  global glob config
  lassign [getOldNewVersions] oldv newv
  set filename $glob(conf_dir)/config$glob(cygwin)-$newv
  Log "Configuration Browser: Saving $filename"
  Try { set fid [open $filename w]
    puts -nonewline $fid [$w get 0.0 end]
    close $fid} "" 1
  catch {destroy $glob(win,left).frame_listb}
  catch {destroy $glob(win,right).frame_listb}
  set glob(select_pry_lr) {}
  # This gets rid of old menus and bindings.  
  array unset config menu,*
  array unset config bind,*
  ReadConfig
  #ForceUpdate
}

proc ConfigBrowserRegenerate { w } {
  global glob
  lassign [getOldNewVersions] oldv newv
  SaveConfig
  set filename $glob(conf_dir)/config$glob(cygwin)-$newv
  set r [catch {set fid [open $filename r]} out]
  if {$r} {
    PopError "Can't open $filename for editing"
    return
  }
  $w delete 0.0 end
  $w insert 0.0 [read -nonewline $fid]
  close $fid
  $w mark set insert 0.0
}
