# -*- tcl -*-
#
# This bit of tcl contains almost all of the inotify setup and use
# code for filerunner.  The code that sets the watch is not included
# since it is short and needs to be called each time a new directory
# is set up.  This code will ONLY be loaded if we are on a Linux system.


proc setUpInotify {} {
  global glob config
  if {$::tcl_platform(os) != "Linux"} {return}
  # We attempt to use the inotify thingy....
  # This could be complicated, but we will keep it simple
  # We only watch the two dirs we are interested in
  # if we catch a notify, we will check and update both
  # We continue to depend on the update time to prune
  # unneeded updates....
  set inotifySo [glob -nocomplain -path $glob(lib_fr)/packages/inotify/ "*.so"]
  if {$inotifySo == {}} {
    # This could be the first run in a tar unarch area, if so see if we have 
    # what is needed...
    set platform $::tcl_platform(machine)
    if {[file exists $glob(lib_fr)/packages/inotify/$platform/pkgIndex.tcl]} {
      foreach fl [glob -nocomplain -tails\
		      -path $glob(lib_fr)/packages/inotify/$platform/ "*"] {
	file link $glob(lib_fr)/packages/inotify/$fl \
	    $glob(lib_fr)/packages/inotify/$platform/$fl
	if {[file extension $fl] == ".so"} {
	  set inotifySo $glob(lib_fr)/packages/inotify/$fl
	}
      }
    }
    set inotifySo [glob -nocomplain -path [file normalize ~/.fr]/ "libinotify*.so"]
  }
  if {$inotifySo != {} && [catch "load $inotifySo inotify" out] == 0 ||\
	[catch {package require inotify} out] == 0} {
    frputs "Loaded inotify version $out "
    set glob(notify,Available) 1
    set glob(inotify_flags) $config(inotify_flags)
    frputs "inotify returns [inotify create notifywatch notecallback]  "
    set glob(notify,watchname) notifywatch
    set glob(notify_id,left) [set glob(notify_id,right) ""]
    foreach lr {left right} {
      ClearCherryPicker $lr
      ClearWatch $lr $glob($lr,pwd)
    }
  } elseif {![file exists $glob(conf_dir)/inotify-message]} {
    set r [catch {open $glob(conf_dir)/inotify-message w} fid]
    if {$r != 0} { 
      PopError "Saving $glob(conf_dir)/inotify-message: $fid"
      return "" 
    }
    set ship [glob -nocomplain -tails -type d\
		  -path $glob(lib_fr)/packages/inotify/ "*"]
    if {$ship == {} && [file exists $glob(lib_fr)/shipInotifyArchs]} {
      source $glob(lib_fr)/shipInotifyArchs
      if {[info exists ::shipInotifyArchs]} {
	set ship $::shipInotifyArchs
      }
    }
    set inot \
	[lindex [lsort -decreasing \
		     [glob -tails -directory\
			  $glob(lib_fr)/Makefiles tcl-inotify-*]] 0]
    puts $fid [_ "Filerunner set up has failed to find a usable tcl inotify\
      library.\n\n" ]
    if {$inotifySo == {} } {    
      puts $fid [_ "Your arch is not one of the ones supported by filerunner as\
      shipped and you have yet to build an inotify package (as described below)\
      and an attempt to load the inotify package from the system area resulted\
      in error: \"%s\"" $out]
    } else {
      puts $fid [_ "Filerunner encoutered error: \"%s\"\n\
      attempting to load: \"%s\"" $out $inotifySo]
    }
    puts $fid [_ "\
      \nThe inotify feature is optional and if no further action is taken this\
      message will not be repeated."]
    puts $fid [_ "\
      \nFilerunner is shipped with the\n\
      %s\
      \nversions of this library.\n" $ship ]
    puts $fid [_ "If you are running on some other architecture and want to\
      use the\
      inotify feature you will have to build the required library.  To do\
      this, you will need the following development packages installed on \
      your system:

      tcl-devel
      gcc       (or some other C compiler)
      make

      There are two approaches you may use to install inotify:
      1.) For just this user of filerunner:
          a.) Navigate to %s using file runner or your
              favorite shell.
          b.) Execute the program/script 'build-inotify' 
               (in filerunner open the command window and type:
               './build-inotify' or in your shell, type the same thing.)
          c.) If the command completes with 'Success', you are done.
              If not, you should have an error message indicating what went
              wrong (most likely a missing tool).

       This puts the inotify files 'pkgIndex.tcl & libinotify*.so' in the
       users .fr directory (~/.fr) where filerunner will find them. You
       may copy them to other users .fr directories if desired.

       2.) For all users on the system (you will need to have root access):
           a. In filerunner navigate to %s
           b. In the opposite panel navigate to an empty directory you will
              use to build the inotify code.
           c. Select the file %s
           d. Click the middle button 'UnArc'
           e. In you favorite shell, navigate to the sub directory that
              step c creates under your empty directory.  The simple
              build is:

              $ ./configure
              $ make
              su
              > make install
              > exit

      './configure --help' gives a whole LOT of extra info if you need it.      

      Inotify is optional and not even available on non-linux systems.\n\n\
      You do NOT need to install it.\n\
      \nThis message is in the file %s. \
      \nOnce inotify is correctly installed you may delete this file.\
      If you delete this file and inotify is not found on the next start up\
      of filerunner, this file will be recreated and displayed. (Thus a\
      test for correct installation of inotify is to delete this file.)\
      If you install it and filerunner does not find it you can look at\
      where filerunner looks by opening a command window in filerunner and\
      typing '%% set auto_path'  This will show where it is looking so you can\
      put the package there.  You can change the install location (method 2)\
      by using the '--prefix' argument in the 'configure' command when \
      building inotify, i.e. ./configure --prefix=/usr/lib\n\n\
      This message may also be seen via the Help menu (Help->inotify)" \
		   $glob(lib_fr)/Makefiles $glob(lib_fr)/Makefiles \
		   $inot $glob(conf_dir)/inotify-message ]
    puts $fid [_ "\nIf you do build an inotify, consider sending it to the author\
     (see the About.. info under the File menu) for inclusion in the next release.\
     Please include the platform name (%s) along with the *.so file.\n\n" \
		   $::tcl_platform(machine)]
    close $fid
    ViewText $glob(conf_dir)/inotify-message
  }
}
#
# The capture code is just for fun.  It is intended to set up hard links
# to newly created files in a subdir of the files creation dir.
# One use for this is to force a temp file to stay around so we can look
# at it at our leisure.
#
# We only monitor the dirs in the two dir panes so there is where we look...
proc setCapture { inst dir } {
  global glob
  if { $inst != "left" && $inst != "right" } {
    return "parm 1 must be 'left' or 'right'"
  }
  set glob(capture_pwd,$inst) $glob($inst,pwd)
  set glob(capture_dir,$inst) $dir
  if { ![file isdirectory $glob($inst,pwd)/$dir] } {
    return "$glob($inst,pwd)/$dir is not a directory"
  }
  defineCapture
}
proc defineCapture {} {
  proc capture { note } {
    global glob
    # If here we have a Creation notify event and we are supposed to hard
    # link the new file to one of the same name in a subdir specified above
    # The parameter here is the 'watch read' result.  We want the file name
    # (7) and the watchid (1) which will tell us which dir ...

    puts "[lindex $note 1] $glob(notify_id,left)"
    if { [lindex $note 1] == $glob(notify_id,left) } {
      set inst left
    } else {
      set inst right
    }
    puts "$inst $glob(capture_pwd,$inst) != $glob(notify,$inst)"
    if { $glob(capture_pwd,$inst) != $glob(notify,$inst) } {
      set glob(capture_dir,$inst) [set glob(capture_pwd,$inst) ""]
      if { $glob(capture_dir,[Opposite $inst]) == "" } {
	return 1
      } else {
	return
      }
    }
    set dir $glob(notify,right)
    # It all looks good
    puts "Trying file link -hard \
	       $glob(notify,$inst)/$glob(capture_dir,$inst)/[lindex $note 7] \
	       $glob(notify,$inst)/[lindex $note 7]"
    catch {file link -hard \
	       $glob(notify,$inst)/$glob(capture_dir,$inst)/[lindex $note 7] \
	       $glob(notify,$inst)/[lindex $note 7] } out
    if {$out == 0 } { 
      puts "success"
    } else {
      puts $out
    }
  }
}
proc capture { note } { }

proc notecallback { {foo 1} } {
  global glob config
  set did 0
  set found {}
  frputs "notecallback entry "
  while {[$glob(notify,watchname) queue]} {
    set out [$glob(notify,watchname) read]
    # watchname read returns 'ID id Flag flags Cookie cookie File file'
    # where the capital element is fixed and the lower case is the var.
    frputs "notecallback  " out
    foreach ou $out {
      set file [lindex $ou 7]
      set reason [lindex $ou 3]
      set lr [expr {$glob(notify_id,left) == \
			[lindex $ou 1] ? {left} : {right} }]
      if {$file != {} } { 
	if { $reason == "n" } {
	  if { [capture $ou] == 1 } {
	    proc capture { note } { }
	  }
	}
	if {[string first $reason "rwaCco"] != -1 } {
	  lappend glob(n_files,$lr) $file
	} else {
	  append found $lr
	  if { $reason == "f"} {puts "inotify queue overflow"}
	}
      }
    }
  }
  # We need an update, but we don't want to do them two quickly and
  # overload things.
  foreach inst {left right} {
    if { $glob(inotify_after,$inst) != {} } {continue}
    if {[set lr [expr {$inst in $found}]] || $glob(n_files,$inst) != {} } {
      set timeval [expr {[clock seconds] - \
			     $glob($inst,lasttime) - \
			     $config(inotify_rate_limit)}]
      if {$timeval >= 0 } {
	catch {after cancel $glob(inotify_after,$inst)}
	set glob(inotify_after,$inst) {}
	ForceNoticeUpdate $inst $lr
#	puts "now $timeval"
      } else {
	#Too fast, slow things down a bit
	set glob(inotify_after,$inst) [after [expr {-$timeval *1000}] \
				      ForceNoticeUpdate $inst $lr]
	frputs "later $timeval [clock seconds] "
      }
    }
  }
}

proc ForceNoticeUpdate {inst allflag } {
  global glob
  catch "after cancel $glob(inotify_after,$inst)"
  set glob(inotify_after,$inst) {}
#  if { $glob(inotify_after,$inst) != {} } { puts "is later $allflag [clock seconds]"}
  frputs "iNotify up date $allflag "
  if {$allflag != -1} {
    set glob($inst,lastmtime) 0
    ListUpdater
    set glob(n_files,$inst) {}
    return
  }
  cherryPickUpdate $inst
  set glob($inst,lasttime) [clock seconds]
}

# The cherryPickUpdate routine updates the files dir list without doing the 
# whole dir list thing.  To do this we need to find the file in the list box.
# Since the listing may be large and a changing file may continue to change,
# we will keep a cache of such files.

proc cherryPickUpdate {args} {
  global glob config cherrycount
  if {! $glob(enableautoupdate)} {   
    trace remove variable glob(enableautoupdate) write cherryPickUpdate
    trace add variable glob(enableautoupdate) write cherryPickUpdate 
    frputs "iNotify Cherry pick update blocked ++++++++++++++++++ "
    return
  } else {
#    puts "cherry pick update"
    trace remove variable glob(enableautoupdate) write cherryPickUpdate 
  }
#  set glob(enableautoupdate) 0
  # this counter just keeps on counting as long as fr lives...
  incr cherrycount
  foreach inst {left right} {
    #puts "CP $inst"
    foreach file $glob(n_files,$inst) {
      #puts "CP $file"
      set index [lsearch -exact -index 0 $glob(n_file_cache,$inst) $file]
      if {$index != -1} {
	# Cache hit, is it a repeat of a file we already updated?
	if {$cherrycount == [lindex $glob(n_file_cache,$inst) $index 2]} {
	  # Yes, just push on...
	  #frputs "iNotify CherryPick repeat "
	  continue
	}
	# NO, change the 'cherrycount' so we will pass the above test if
	# this file is in the list more than once.
	# According to Murphy, we may have all these reset while we
	# are trying to update things.  If this falls apart, just get out.
	if { [catch "lreplace glob(n_file_cache,$inst)  $index $index \
	                [list $file \
		        [lindex $glob(n_file_cache,$inst) $index 1]\
		        $cherrycount]" ] } {
	  frputs "iNotify CP fall apart "
	  break
	}
	# Here we change the meaning of 'index' above it is the entry number
	# in the cache, after the following, 
	# it is the index in 'glob(l/r,lvfile)'
	lset glob(n_file_cache,$inst) $index 2 $cherrycount
	set index [lindex $glob(n_file_cache,$inst) $index 1]
      } else {
	set index [lsearch -exact -index 1 $glob($inst,filelist) $file]
	frputs "iNotify CP $file  " index
	if {$index == -1} {
	  # file not found in the master list for this side
	  break
	}
	lappend glob(n_file_cache,$inst) [list $file $index $cherrycount]
      }
      #	puts "cache $glob(n_file_cache,$inst)"
      # size mtime mode usergroup link nlink atime ctime
      frputs "iNotify CherryPick doing lstat on  " file index
      if {[catch {file lstat "$glob(notify,$inst)/$file" statinfo} ]} continue
      frputs "iNotify CherryPick file $file sz $statinfo(size) "
      set size $statinfo(size) 
      set mode [GetStringFromMode $statinfo(mode)]
      set usergroup [GetUidGidString $glob(notify,$inst)/$file \
			 $statinfo(uid) $statinfo(gid)]
      set nlink $statinfo(nlink)
      set mtime $statinfo(mtime)
      set atime $statinfo(atime)
      set ctime $statinfo(ctime)
      set lr $inst
      if {$glob(notify,left) == $glob(notify,right)} {
	set lr 	[list {left} {right}]
      }
      foreach side  $lr {
	if {$glob(notify,$inst) != $glob($side,pwd) } {
	  frputs "notify pwd failure $glob(notify,$inst) != $glob($lr,pwd) \n
                but glob(notify,left) & right == "
	  continue
	}
	lassign [$glob(listbox,$side).file yview] ypos yypos
	foreach lbentry $config(ListBoxColumns,$side) {
	  set flist [lindex $lbentry 0]
	  if {$flist == "file" || $flist == "slink" } continue
	  
	  lset glob($inst,lv$flist) $index  [eval $glob(lbscript,$flist)]
	}
      }
    }
    set glob(n_files,$inst) {}
  }
#  set glob(enableautoupdate) 1
}
