
proc ReplaceTearoff {w menu tearoff} {
# this code builds a replacement tearoff menu that does NOT deiconify
# when the 'master' window loses focus.
# It should be called by the '-tearoffcommand' passing the same 3 prams it
# got assuming that the parent window was passed. 
# Note: w is used to define what the tearoff should appear above.  In most
#       cases "." is a good choice.  Also, when "w" is destroyed this menu
#       will be destroyed.  This, of course, only matters when "w" is not ".". 
# (In fact this could be the -tearoffcommand)
#
# Caution, this code does not handle Radio or Check  entries
#
# No external dependencys
#
  global tearoffBuildCount
  lassign [winfo pointerxy $tearoff] x y
  set wn [toplevel .toplevelTear_[incr tearoffBuildCount] -class ReplaceTear ]
  wm title $wn [wm title $tearoff]
  wm geometry $wn [winfo geometry $tearoff]
  # the following fixes the M$ system wanting to put the tearoff at +0+0
  # the above picked up the size...
  wm geometry $wn "+$x+$y"
  wm minsize $wn 92 1      ; # this is to keep the window large enough to move
  set wt [text $wn.text -undo true]
#  puts "geo $w $tearoff to $wt [winfo geometry $tearoff]"
  wm transient $wn $w
  bind $w <Destroy> "+destroy $wn"
#  puts "building $wn for $w"
#  bind $wn <KeyPress> {puts "keypress"}
  foreach option [$tearoff configure] {
    if {[llength $option] < 5 } {continue}
    lassign $option optionkey d1 d2 d3 value
    set r 0
#    puts "replacetearoff $optionkey $value"
    switch $optionkey {
      -activebackground  {set activebg $value}
      -relief            {set relief   $value}
      -activeborderwidth {set border   $value}
      -background        {set bg       $value}
      -title              -
      -type               -
      -tearoffcommand     -
      -activeforeground   -
      -disabledforeground -
      -selectcolor        -
      -tearoff            -
      -postcommand      { continue}
      default {
	set r [catch "$wt configure $optionkey [list $value]" ho]
      }
    }
    if {$r != 0} {
      frputs "replacetearoff: configure $optionkey $value returned $ho "
    }
  }
  if {[catch "$wt configure -background $bg" ho] != 0} {
    frputs "replacetearoff: configure -background $bg returned $ho "
  }
  $wt tag configure right -justify right
  for {set i 1} {$i <= [$tearoff index end]} {incr i} {
    set tearoffent [$tearoff entryconfigure $i]
    if {[llength $tearoffent] < 3} {
      $wt insert end "-------- \n"
      continue
    }
    set cascade {}
    set thisbg $bg
    set thisactivebg $activebg
    foreach option $tearoffent {
      lassign $option optionkey d1 d2 d3 value
      if {$value == {} } {
	continue
      }
#      puts "$optionkey >$value<"
      switch $optionkey {
	-label   { 
	  $wt insert end "$value" tag$i
	  if {$cascade != {} } {
	    $wt insert end "  ->" right
	  }
	  $wt insert end "\n"
	}
	-menu -
	-command {
	  if {$optionkey == "-menu"} {
	    $wt tag bind tag$i <1> \
              "tk_popup $value \[winfo pointerx $wt] \[winfo pointery $wt]"
	    set cascade "--->"
	  } else {
	    $wt tag bind tag$i <1>  $value
	  }
	  $wt tag configure tag$i -spacing1 4
	  $wt tag bind \
	      tag$i <Enter> "$wt tag config tag$i \
                              -background $thisactivebg \
                              -borderwidth $border \
                              -relief $relief"
	  $wt tag bind \
	      tag$i <Leave> "$wt tag config tag$i \
                             -background $thisbg \
                             -borderwidth 0"
	  $wt tag config tag$i -background $thisbg -borderwidth 0
	}
	-activebackground { set thisactivebg $value}

	-background       {set thisbg $value}
	default {continue }
      }
    }
  }
  $wt edit reset
  $wt edit modified 0
  bind $wt <<Modified>> "$wt edit undo"
  bind $wt <Button> {break}
  bind $wt <Motion> {break}
  pack $wt -expand 1 -fill both
  destroy $tearoff
  update
  wm deicon $wn
}
