#
# This is a 'smart' take off on tk_dialog.  It is set up
# so that we can use colors in the text, center the window
# above the master (and keep it above), provide variable
# number of buttons with out changing the call sequence
# (we allow unused buttons to be specified and then not used).
#
#  parms
#  win    the desired window name (if one already exists it is destroyed)
#  master win will be centered above master and will be destroyed when
#         master is.
#  title  title for win
#  text   a list of strings for the windows query area
#         the strings will be concationated in the window
#         the even entries will be in standard text and
#         the odd entries in 'select' text
#         empty strings may be used to switch as needed.
#         The window height will be as needed up to 30 lines.
#         If more that 30 lines are needed a scroll bar will appear.
#         You can change this, see 'scroll' under args.
#         This argument is ignored if you code 'textfun' in the 'args'
#         options below.
#         If the resulting text part of the widget is empty, i.e. text is {}
#         and 'textfun' does not add text, the text part is removed from the
#         widget.
#  default button number (starting from 0) of the default button
#          -1 implies no default.  A carrage return in the window
#          will return this number.
#  buttonCount If the number of buttons to build.
#              > 0  Build 'buttonCount' buttons as specified by 'buttons'.
#              =  0 Build a frameless text only window.  This window will
#                   need  to be destroyed by the caller or destruction of
#                   the master  or by mouse <1> click.  The window will by
#                   default flash in two ways:
#                       1) a flat border and configured bg color
#                       2) a solid border and a 'flash' bg color
#                   There are 4 configure options for the flash.  A flash
#                   time of '0' will turn off flashing (as will flashon == 0).
#                   See the options below under 'args'.          
#  buttons a list of button labels.  Should be as long as 'buttonCount'
#          but may be longer (if shorter than button count, blank buttons
#          are created). Each button label may be a list as follows:
#          {name {'other'}}             just a button, pressing will
#                                       close the window and return this
#                                       buttons number.
#          {name {-command {script} 'other'}      Button press will run script.
#          {name {-variable globalvar 'other'}}   Button becomes a checkbutton 
#                                                 with variable globalvar
#                                                 which will assumed values
#                                                 'true' or 'false'.
#          {{name} {-textvariable globalvar 'other'}} Builds an 'entry' window
#                                                 below the text window with
#                                                 'globalvar' as its
#                                                 '-textvariable'.
#                                                 Bindings for <Return> and
#                                                 <KB_Return>  will close the
#                                                 window and return the button
#                                                 number. <Escape> will close
#                                                 the window and return -1.
#                                                 'name' (if not "") will create
#                                                 a label to the left of the
#                                                 entry window.
#          {{name} {-text {text} 'other'}}        Builds a label only entry
#                                                 below the the text window.
#                                                 'name' is not used and may be
#                                                 "". The label will span the
#                                                 whole width of the window.
#          In all cases 'other'  may be empty or may contain other options for
#          the given widget (-width xx might be good for entry widgets). 
#          In the first case 'other' must not start with '-command, 
#          -variable or -textvariable.
#
#  Groups  There are three groups, each of which forms a part of the window
#          Group 1 is the text window, 2 consists of the -textvariable and
#          -text "buttons" and group 3 is the rest of the "buttons".
#          By default these appear in order, however, the "arrange" option
#          (see below may be used to change the order).
#
#  args    (optional) one or more lists that contains info on extra window
#          decorations this will be used with the text window (which is of
#          type text and may be any option a text window supports. Most
#          likely this  will be used to set up select colors. Useful things:
#          -selectforeground         color      (default  red)
#          -inactiveselectbackground color      (default yellow)
#          -background               color      (default white)
#          -foreground               color      (default black)
#          -font                     font       (TkCaptionFont)
#          -height                   nnn        (as needed by 'text' but > 30
#                                               will create a scrolled window
#                                               of 30 lines)
#          -flash                    <on/off time in ms> (500 (0 turns off))
#          -flashborderwidth         n          (widget default, i.e. 1)
#          -flashcolor               color      (red)
#          -flashon                  bool       (0 if button count > 0)
#          maxw                      int        over ride max window width pixals
#                                               (default is master window width)
#          tag                       {tag config} (no default)
#          scroll                    n          (default 30) Height >= 'n' 
#                                               will limit to n
#                                               with a scroll bar
#          textfun                   {script}   script is post pended with the
#                                               text window name and called 
#                                               instead of putting 'text' in
#                                               the window. This function can
#                                               set up tags on the entries. These
#                                               tags can bind to funtions which can
#                                               force the window to close and return "x"
#                                               by writting "x" to ::smart_dialogBut$w 
#                                               where $w is the text window name.
#          bind                      {script}   to bind to buttons or text
#                                               button names are 'win.no' where
#                                               no is the button number. The
#                                               text widget is 'win.text' 
#          debug                      {}        causes return with out waiting
#                                               for input/etc.
#          position                  {script}   This function is called to
#                                               position the smart_dialog window.
#                                               Two arguments are passed, the
#                                               window name and the master window
#                                               name. By default "centerWin" is
#                                               called.                  
#          stackbuttons              {}         arrange buttons in one column 
#                                               rather than the default row.
#          arrange                   {num}      "num" is a list of three
#                                               numbers which can change the 
#                                               order of the "groups" (see
#                                               above. Default is {1 2 3}.
#                                               {3 2 1} would put the buttons
#                                               on top, and text on the bottom.
#                                               {2 1 3} puts the -text and
#                                               -textvariable on top, text
#                                               in the middle and buttons on 
#                                               the bottom. These numbers become
#                                               the "row" numbers for each group.
#
#     The tag configure allows you to configure a tag (named tag) that is used
#     on every line of text. One example is:
#          {tag {config tag -justify center}}
#     which will cause lines to be centered in the window.
#
#  smart_dialog returns the number of the button pressed or -1 if the window is
#               destroyed before its time.
#
#          smart_dialog will build a window with a minimum width of that 
#          required for the specified buttons and a maximum width of 
#          what ever the masters width is (as long as it is greater than
#          the above minimum.
#
#          This widgit uses centerWin
#
proc smart_dialog {win master title text {default 0} {buttonCount 0} \
		       {buttons {} } args} {
  destroy $win
  set w $win.text
  catch {after cancel $::smart_dialogAfter$w}
  toplevel $win -class Dialog
  wm iconname $win Dialog
  wm title $win $title
  wm protocol $win WM_DELETE_WINDOW "set ::smart_dialogBut$w -1"
  bind $win <Destroy> "set ::smart_dialogBut$w -1"
  bind $master <Destroy> "+destroy $win"
#  set master [regsub {(^\.[^\.]+).*$} $master {\1}]
#  puts "master is $master"
#  puts "$text"
  set color red
  set flash 500
  set position {centerWin}
  set flashOn [expr {$buttonCount > 0 ? 0 : 1}]
  lassign {0 1 2} g1 g2 g3
  text $w \
      -height 1 \
      -wrap word \
      -setgrid true 
  frame $win.mid
  # For reasons we do not understand, this must be here or we will not
  # be able to get this windows width to drive the master window width
  # We would prefer to have included these options with the -row, below.
  grid $w -column 0 -sticky nsew

  if {$buttonCount > 0 } {
    frame $win.bot
    $win.bot configure -relief raised -bd 1
    #grid $win.bot -row $g3 -columnspan 2 -sticky sew 
    grid anchor $win.bot center
  } else {
    wm withdraw $win
    wm overrideredirect $win 1
  }
  set width 0
  set entrywindows {}
  update
  lassign {0 1 2} g1 g2 g3
  lassign [split [winfo geo $master] "x"] maxw
  #puts "$maxw [winfo geo $master]"
  set maxheight 30
  frputs "smart_dialog window $win "
  set grid {}
  for {set co 0} {$co < $buttonCount} {incr co} {
    lassign [lindex $buttons $co] name extra
    lassign $extra option
    # we don't want to require double brackes for button
    # names if s/he is not trying to include an option so..
    if {[string range $extra 0 0] != "-"} {
      set name [lindex $buttons $co]
      set extra {}
    }
    set r 0
    switch $option {
      -text {
	label $win.$co 
 	set r [catch "$win.$co config $extra" ho]
	grid $win.$co -in $win.mid -row $co -column 0 -columnspan 2 -sticky ew
      }
      -variable {
	checkbutton $win.$co -text $name
	set r [catch "$win.$co config $extra" ho]
	$win.$co config -borderwidth 3 -relief raised
        $win.$co config -anchor w
 	lappend grid $co 
	incr width 3
	if {$default == $co} {
	  set default -1
	}
     }
      -textvariable {
	lappend entrywindows $co
	entry $win.$co 
	label $win.label$co -text $name
	grid $win.label$co -in $win.mid -row $co -column 0 -sticky e
	grid $win.$co -in $win.mid -row $co -column 1 -sticky ew
	bind $win.$co <Return> "set ::smart_dialogBut$w $co"
	bind $win.$co <KP_Enter> "set ::smart_dialogBut$w $co"
	bind $win.$co <Escape> "set ::smart_dialogBut$w -1"
 	set r [catch "$win.$co config $extra" ho]
	$win.$co icursor [string length [$win.$co get]]
     }
    -command -
      default {
	button $win.$co \
	    -text $name \
	    -command "set ::smart_dialogBut$w $co"
	$win.$co config -borderwidth 3 -relief raised
	$win.$co config -default [expr {$co == $default ? "active" : "normal"}]
	set r [catch "$win.$co config $extra" ho]
	lappend grid  $co
      }
    }
    if {$r != 0} {frputs "button command >$win.$co config $extra Returns $ho "}
  }
  foreach decor $args {
    if {$decor != {}} {
      foreach {op val} $decor {
	set r 0
	switch $op {
	  -flash {set flash $val}
	  -flashcolor {set color $val}
	  -flashborderwidth {$w config -borderwidth $val}
	  -flashon {set flashOn $val}
	  maxw    {set maxw $val}
	  bind    {set r [catch "bind $val" ho]}
	  tag     {set r [catch "$w $op $val" ho]}
	  debug   {set debug 1}
	  scroll  {set maxheight $val}
	  textfun {set textfun $val}
	  position {set position $val}
	  stackbuttons {set stack 1}
	  arrange {
	    lassign $val g1 g2 g3
	    # set error if not numbers
	    foreach gn [list $g1 $g2 $g3] {
	      if {! [regexp {^[0-9]+$} $gn]} {
		set r 1
		set ho "$gn is not a numeric"
		lassign {1 2 3} g1 g2 g3
g		break
	      }
	    }
	  }
	  default {
	    set r [catch "$w config $op \"$val\"" ho]
	    #	  puts "$r $op $val $hu"
	  }
	}
	if {$r != 0} {frputs "args $w config \{$op $val\}  Returns $ho "}
      }
    }
  }
  grid $w -column 0 -row $g1 -sticky nsew
  grid rowconfig $win $g1 -weight 1
  grid columnconfig $win 0 -weight 1
  grid columnconfig $win.mid 1 -weight 1
  grid $win.mid -in $win -row $g2 -sticky ewns
  if {$buttonCount > 0 } {
    grid $win.bot -row $g3 -sticky sew 
  }

  # grid the check and normal buttons
  foreach cmd $grid {  
    set rs [expr {[info exists stack] ? "-column 0 -sticky ew -row" :\
		      "-row 0 -sticky news -column"}]
    grid $win.$cmd -in $win.bot {*}$rs $cmd -padx 5 -pady 5
  }

  if {$buttonCount > 0 } {
    for {set co 0} {$co < $buttonCount} {incr co} {
      set butfont  [lindex [$win.$co config -font] 4]
      # It seems the text is padded with 3 spaces front & back & we 
      # left and right pad by 5 and have borders of 3 each side -> 16
      if {$co ni $entrywindows} {
	set thiswidth [expr {[font measure $butfont \
				  "   [lindex $buttons $co 0]   "] + 16}]
	if {[info exists stack]} {
	  set width [expr {max($width,$thiswidth)}]
	} else {
	  incr width $thiswidth
	}
      } else {
	set maxcw [expr {$maxw / [font measure $butfont "0"]}]
	if {[lindex [$win.$co config -width] 4] > $maxcw} {
	  $win.$co config -width $maxcw
	}
      }
    }
  }
  # width is now the min width we want to use...in pixels
  if {[info exist textfun] } {
    eval "$textfun $w"
  } else {
    foreach {std hig} $text {
      $w insert insert "$std"  tag 
      $w insert insert "$hig" [list  tag sel col.first col.last]
    }
  }
  set tmp [split [$w get 1.0 end] "\n"]
  set font [lindex [$w config -font] 4]
  set mx 0
  # we use butWidth as the min window width, the masters width as the max
  set maxw [expr {max($maxw,$width)}]
  set maxfudge [expr {($maxw * 9) / 10}]
  set charsz [font measure $font "0"]
#  puts "$font $maxw $tmp"
  set height -1
  foreach line $tmp {
    incr height
    set thisw [font measure $font -displayof $w $line]
    #puts "$height $thisw $width $maxw $line"
    if {$thisw > $maxw} {
      incr height [expr {$thisw / $maxfudge}]
#      puts "$height [expr {$thisw / $maxfudge}]"
      set width $maxw
    } else {
      set width [expr {max($width,$thisw)}]
    }
  }
#  puts "[$w config -height]"
  set height [expr {max($height,[lindex [$w config -height] 4])}]
  if {$height > $maxheight} {
    # lets use a scroll bar to control this...
    set height $maxheight
    scrollbar $win.scroll  -relief sunken -command "$w yview"
    $w config -yscrollcommand "$win.scroll set"
    grid $win.scroll -column 1 -in $win -row $g1 -sticky nse
  }
  #puts "setting height $height width [expr {($width / $charsz) +1}]"
  $w config -height $height -width [expr {($width / $charsz) +1}] 
  if {$tmp == {{} {}} } {
    # remove the text window if it is empty
    grid remove $w
  }
#  $w config -state disabled
  if {$default >= 0} {
    bind $w <Return> " 
	    $win.$default configure -state active -relief sunken
	    update idletasks
	    after 100
	    set ::smart_dialogBut$w $default
	"
  }
#  bind $w <Button> {break}
  bind $w <KeyRelease> {break}
  bind $w <KeyPress> {break}
  bind $w <Motion> {break}
  bind $w <Leave> {break}
  bind $w <Enter> "focus $w \; break"
#  bind $w <<Modified>> "$w edit undo"
  catch {wm deiconify $master}
  update 
  eval $position $win $master
  #centerWin $win $master
  wm transient $win $master
  if {$buttonCount < 1} {
    bind $w <1> "destroy $win \; \
                 catch \"after cancel \[set ::smart_dialogAfter$w]\" \; break"
    if {$flash != 0 && $flashOn} {
      set bgcolor [lindex [$w config -bg] 4]
      smart_dialog_flash $w $flash $color $bgcolor 1
    }
    raise $win
    wm deiconify $win
    eval $position $win $master
    return
  }
  set oldFocus [focus]
  set oldGrab [grab current $w]
  if {$oldGrab != ""} {
    set grabStatus [grab status $oldGrab]
    grab release $oldGrab
  }
  update
  if {$default != "-1"} {
    focus $win.$default
  } else {
    focus $w
  }
  catch {after cancel [set ::smart_dialogAfter$w]}
  if {$flash != 0 && $flashOn} {
    set bgcolor [lindex [$w config -bg] 4]
    smart_dialog_flash $w $flash $color $bgcolor 1
  }
  if {[info exists debug] } {return -2}

  vwait ::smart_dialogBut$w
  catch {focus $oldFocus}
  if {$oldGrab != ""} {
    if {$grabStatus == "global"} {
      grab -global $oldGrab
    } else {
      grab $oldGrab
    }
  }
  catch "set ::smart_dialogBut$w" ex
  catch "destroy $win"
  catch "unset ::smart_dialogBut$w"
  return $ex
}

proc smart_dialog_flash {w flash color bgcolor onoff} {
  catch {after cancel [set ::smart_dialogAfter$w]}
  if {[winfo exists $w]} {
    if {$onoff} {
      set onoff 0
      $w config -relief solid
      $w config -bg $color

    } else {
      set onoff 1
      $w config -relief flat
      $w config -bg $bgcolor
    }
    set ::smart_dialogAfter$w \
	[after $flash "smart_dialog_flash $w $flash $color $bgcolor $onoff"]
  }
}
